{ COPYRIGHT 2008 Michael Van Canneyt und Toolbox Verlag }
unit cdcovers;

{$mode objfpc}{$H+}
{ $define usedelegation}
interface

uses
  Classes, SysUtils, fpcddb, lclintf, graphtype,graphics, contnrs, fpobserver;

Type
  TCDDisc = Class;
  TTrackList = Class;

  { TCDCoverTrack }

Type
  TCDCoverTrack = Class(TObservedCollectionItem)
  private
    FDuration: TDateTime;
    FExtra: String;
    FPerformer: String;
    FTitle: String;
    FYear: Integer;
    function GetCaption: String;
    function GetPerformer: String;
    procedure SetDuration(const AValue: TDateTime);
    procedure SetExtra(const AValue: String);
    procedure SetPerformer(const AValue: String);
    procedure SetTitle(const AValue: String);
    procedure SetYear(const AValue: Integer);
  Protected
    Procedure Changed;
  Public
    Procedure Assign(Source : TPersistent); override;
  Published
    property Caption : String Read GetCaption;
    Property Title : String Read FTitle Write SetTitle;
    Property Performer : String Read GetPerformer Write SetPerformer;
    Property Extra : String Read FExtra Write SetExtra;
    Property Duration : TDateTime Read FDuration Write SetDuration;
    Property Year : Integer Read FYear Write SetYear;
  end;

  { TCDCoverTracks }

  TCDCoverTracks = Class(TCollection)
  private
    FDisc : TCDDisc;
    function GetT(AIndex : Integer): TCDCoverTrack;
    procedure SetT(AIndex : Integer; const AValue: TCDCoverTrack);
  Public
    Property Disc : TCDDisc Read FDisc;
    Function AddTrack(Const ATitle,AExtra : String; ADuration : TDateTime) : TCDCoverTrack;
    Function AddTrack(Const ATitle,AExtra : String) : TCDCoverTrack;
    Function AddTrack(Const ATitle : String) : TCDCoverTrack;
    Property Track[AIndex : Integer] : TCDCoverTrack Read GetT Write SetT; default;
  end;

  { TCDDisc }

  TCDDisc = Class(TObservedComponent)
  private
    FComment: String;
    FPerformer: String;
    FTitle: String;
    FTracks: TCDCoverTracks;
    FYear: Integer;
    function GetCaption: String;
    procedure SetComment(const AValue: String);
    procedure SetPerformer(const AValue: String);
    procedure SetTitle(const AValue: String);
    procedure SetTracks(const AValue: TCDCoverTracks);
    procedure SetYear(const AValue: Integer);
  Public
    Constructor Create(AOwner : TComponent); override;
    Destructor Destroy; override;
  Published
    property Caption : String Read GetCaption;
    Property Title : String Read FTitle Write SetTitle;
    Property Performer : String Read FPerformer Write SetPerformer;
    Property Tracks : TCDCoverTracks Read FTracks Write SetTracks;
    Property Extra : String Read FComment Write SetComment;
    Property Year : Integer Read FYear Write SetYear;
  end;

  TPageType = (ptBookletFront,ptBookletBack,ptInlay,ptDiscLabel);
  TPageTypes = Set of TPageType;

  { TCoverObject }

  { TMemento }

  TMemento = Class(TObject)
    Constructor CreateMemento; virtual;
  end;
  TMementoClass = Class of TMemento;

  { TCoverComponent }

  TCoverComponent = Class(TObservedComponent)
  Protected
    Class Function GetMementoClass : TMementoClass; virtual;
    Procedure DoApplyMemento(AMemento : TMemento); virtual;
    Procedure DoFillMemento(AMemento : TMemento); virtual;
  Public
    Function GetMemento : TMemento;
    Procedure ApplyMemento(AMemento : TMemento);
  end;

  TCoverObject = Class(TCoverComponent)
  private
    FDims : Array[1..4] of Double;
    FParent : TComponent;
    function GetDim(AIndex: integer): Double;
    procedure SetDim(AIndex: integer; const AValue: Double);
  Protected
    Class Function GetMementoClass : TMementoClass; override;
    Procedure DoApplyMemento(AMemento : TMemento); override;
    Procedure DoFillMemento(AMemento : TMemento); override;
    procedure DrawObject(ACanvas : TCanvas; PageRect : TRect; ADPI : Integer); virtual; abstract;
  Public
    Constructor Create(AOwner : TComponent); override;
    Function HavePoint(X,Y : Double) : Boolean;
    Function GetObjectRect(ACanvas : TCanvas; Const PageRect : TRect; ADPI : Integer) : TRect;
    Function GetObjectRect(ACanvas : TCanvas; Const PageOrigin : TPoint; ADPI : Integer) : TRect; virtual;
    procedure SetParentComponent(Value: TComponent); override;
    Function GetParentComponent : TComponent; override;
  Published
    Property Width : Double Index 1 Read GetDim Write SetDim;
    Property Height : Double Index 2 Read GetDim Write SetDim;
    Property Left : Double Index 3 Read GetDim Write SetDim;
    Property Top : Double Index 4 Read GetDim Write SetDim;
  end;

  TCoverObjectClass = Class of TCoverObject;


  { TCoverPage }

  TCoverPage = Class(TCoverComponent)
  private
    FHeight: Double;
    FWidth: Double;
    FObjects : TFPList;
    function GetObject(AIndex : Integer): TCoverObject;
    function GetObjectCount: Integer;
    procedure SetHeight(const AValue: Double);
    procedure SetWidth(const AValue: Double);
  protected
    Class Function GetMementoClass : TMementoClass; virtual;
    Procedure DoApplyMemento(AMemento : TMemento); virtual;
    Procedure DoFillMemento(AMemento : TMemento); virtual;
    procedure DrawBookletBack(ACanvas: TCanvas; PageRect: TRect; ADPI: Integer); virtual;
    procedure DrawBookletFront(ACanvas: TCanvas; PageRect: TRect; ADPI: Integer);virtual;
    procedure DrawDiscLabel(ACanvas: TCanvas; PageRect: TRect; ADPI: Integer);virtual;
    procedure DrawInlay(ACanvas: TCanvas; PageRect: TRect; ADPI: Integer);virtual;
    function GetPageType: TPageType; virtual; abstract;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
  Public
    Constructor Create(AOwner : TComponent); override;
    Destructor Destroy; override;
    procedure Clear;
    procedure DrawObject(AObject : TCoverObject; ACanvas : TCanvas; Origin : TPoint; ADPI : Integer);
    procedure DrawPage(ACanvas : TCanvas; Origin : TPoint; ADPI : Integer);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    Function GetObjectAt(X,Y : Double) : TCoverObject;
    Property Objects[AIndex : Integer] : TCoverObject Read GetObject;
    Property ObjectCount : Integer Read GetObjectCount;
    Property PageType : TPageType Read GetPageType;
  Published
    Property Width : Double Read FWidth Write SetWidth;
    Property Height : Double Read FHeight Write SetHeight;
  end;

  { TBookletFrontPage }

  TBookletFrontPage = Class(TCoverPage)
  protected
    function GetPageType: TPageType; override;
  end;

  { TBookletBackPage }

  TBookletBackPage = Class(TCoverPage)
  protected
    function GetPageType: TPageType; override;
  end;

  { TInlayPage }

  TInlayPage = Class(TCoverPage)
  Private
    FDepth: Double;
    procedure SetDepth(const AValue: Double);
  Protected
    function GetPageType: TPageType; override;
    procedure DrawInlay(ACanvas: TCanvas; PageRect: TRect; ADPI: Integer);override;
  Public
    Constructor create(AOwner : TComponent); override;
  Published
    Property InlayDepth : Double Read FDepth Write SetDepth;
  end;

  { TDiscLabelPage }

  TDiscLabelPage = class(TCoverPage)
  Private
    FRadius1: Double;
    FRadius2: Double;
    function GetRadius: Double;
    function GetRadiusStored: boolean;
    procedure SetRadius1(const AValue: Double);
    procedure SetRadius2(const AValue: Double);
  Protected
    function GetPageType: TPageType; override;
    procedure DrawDiscLabel(ACanvas: TCanvas; PageRect: TRect; ADPI: Integer);override;
  Public
    COnstructor Create(AOwner : TComponent); override;
  Published
    Property Radius1 : Double Read FRadius1 Write SetRadius1;
    Property Radius2 : Double Read GetRadius Write SetRadius2 stored GetRadiusStored;
  end;

  { TCDCover }

  TCDCover = Class(TCoverComponent)
  Private
    FDescription: TStrings;
    FDiscs : TFPList;
    FPages : TFPList;
    FPerformer: String;
    FTitle: String;
    function GetDisc(AIndex : Integer): TCDDisc;
    function GetDiscCount: Integer;
    function GetPage(AIndex : Integer): TCoverPage;
    function GetPageCount: Integer;
    function GetPerformer: String;
    function GetTitle: String;
    procedure SetDescription(const AValue: TStrings);
    procedure SetPerformer(const AValue: String);
    procedure SetTitle(const AValue: String);
  Protected
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
  Public
    Constructor Create(AOwner : TComponent); override;
    Destructor Destroy; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Clear;
    Procedure LoadFromStream(Stream : TStream); virtual;
    Procedure SaveToStream(Stream : TStream);virtual;
    Procedure LoadFromFile(Const AFileName : String);
    Procedure SaveToFile(Const AFileName : String);
    Procedure GetDiscs(List : TList);
    Procedure GetPages(List : TList);
    Function AddDisc : TCDDisc;
    Function AddPage(APageType : TPageType) : TCoverPage; virtual;
    Function HavePageType(APageType : TPageType) : Boolean;
    Property Disc[AIndex : Integer] : TCDDisc Read GetDisc;
    Property DiscCount : Integer Read GetDiscCount;
    Property Page[AIndex : Integer] : TCoverPage Read GetPage;
    Property PageCount : Integer Read GetPageCount;
  Published
    Property Description : TStrings Read FDescription Write SetDescription;
    Property Title : String Read GetTitle Write SetTitle;
    Property Performer : String Read GetPerformer Write SetPerformer;
  end;


  TElementType = (etTrackNo,etPerformer,etTitle,etDuration);
  { TTrackelement }

  TTrackelement = Class(TObservedPersistent)
  private
    FAlignment: TAlignment;
    FFormat: String;
    FParentFont: Boolean;
    FPosition: Integer;
    FVisible: Boolean;
    FTrackList : TTrackList;
    FFont: TFont;
    FWidth: Double;
    FElementType : TElementType;
    function GetElementName: String;
    function GetFontStored: boolean;
    procedure ParentFontChanged;
    procedure SetAlignMent(const AValue: TAlignment);
    procedure SetFont(const AValue: TFont);
    procedure SetFormat(const AValue: String);
    procedure SetParentFont(const AValue: Boolean);
    procedure SetPosition(const AValue: Integer);
    procedure SetVisible(const AValue: Boolean);
    procedure SetWidth(const AValue: Double);
    Procedure FontChanged(Sender : TObject);
  Protected
    Procedure Changed; override;
  Public
    Constructor Create; override;
    Destructor Destroy; override;
    procedure Assign(Source : TPersistent); override;
    Property ElementType : TElementType Read FElementType;
  Published
    Property ParentFont : Boolean Read FParentFont Write SetParentFont;
    Property Font    : TFont Read FFont Write SetFont stored GetFontStored;
    Property Format  : String Read FFormat Write SetFormat;
    Property Visible : Boolean Read FVisible Write SetVisible;
    Property Width : Double Read FWidth Write SetWidth;
    Property Alignment : TAlignment Read FAlignment Write SetAlignMent;
    Property Position : Integer Read FPosition Write SetPosition;
  end;

  { TTrackList }

  TTrackList = Class(TCoverObject)
  private
    FDisc: TCDDisc;
    FFont: TFont;
    FTrackElements : Array[0..3] of TTrackElement;
    Function GetElementText(E: TTrackElement; T: TCDCoverTrack) : string;
    function GetT(Index : Integer): TTrackElement;
    function GetTrackElement(AIndex: integer): TTRackElement;
    procedure SetDisc(const AValue: TCDDisc);
    procedure SetFont(const AValue: TFont);
    procedure SetTrackElement(AIndex: integer; const AValue: TTRackElement);
  Protected
    Class Function GetMementoClass : TMementoClass; override;
    Procedure DoApplyMemento(AMemento : TMemento); override;
    Procedure DoFillMemento(AMemento : TMemento); override;
    Procedure Reposition(FElement : TTrackElement; NewPos : Integer);
    procedure DrawObject(ACanvas : TCanvas; PageRect : TRect; ADPI : Integer); override;
    Procedure Loaded; override;
  Public
    Constructor Create(Aowner : TComponent); override;
    Destructor Destroy; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    Property Elements [Index : Integer] :TTrackElement Read GetT;
  Published
    Property Font      : TFont Read FFont Write SetFont;
    Property Disc      : TCDDisc Read FDisc Write SetDisc;
    Property TrackNr   : TTRackElement Index Ord(etTrackNo) Read GetTrackElement Write SetTrackElement;
    Property Performer : TTRackElement Index Ord(etPerformer) Read GetTrackElement Write SetTrackElement;
    Property Title     : TTRackElement Index Ord(etTitle) Read GetTrackElement Write SetTrackElement;
    Property Duration  : TTRackElement Index ord(EtDuration) Read GetTrackElement Write SetTrackElement;
  end;

  TShapeType = (stHorizontal,stVertical,stDiagonal1,stDiagonal2,stEllipse,stRectangle,stRoundedRectangle);

  { TShapeObject }

  TShapeObject = Class(TCoverObject)
  private
    FPenColor: TColor;
    FFillColor: TColor;
    FPenWidth: Integer;
    FRHR: Double;
    FRVR: Double;
    FShapeType: TShapeType;
    FTransParent: Boolean;
    procedure SetFillColor(const AValue: TColor);
    procedure SetPenColor(const AValue: TColor);
    procedure SetpenWidth(const AValue: Integer);
    procedure SetRHR(const AValue: Double);
    procedure SetRVR(const AValue: Double);
    procedure SetShapeType(const AValue: TShapeType);
    procedure SetTransParent(const AValue: Boolean);
  Protected
    Class Function GetMementoClass : TMementoClass; override;
    Procedure DoApplyMemento(AMemento : TMemento); override;
    Procedure DoFillMemento(AMemento : TMemento); override;
    procedure DrawObject(ACanvas : TCanvas; PageRect : TRect; ADPI : Integer); override;
  Public
    Constructor Create(AOwner : TComponent); override;
    procedure DrawShape(ACanvas: TCanvas; R: TRect; ADPI: Integer); virtual;
  Published
    Property Transparent : Boolean Read FTransParent Write SetTransParent;
    Property PenColor : TColor Read FPenColor Write SetPenColor default clBlack;
    property PenWidth : Integer Read FPenWidth Write SetpenWidth Default 1;
    Property FillColor : TColor Read FFillColor Write SetFillColor default clWhite;
    Property ShapeType : TShapeType Read FShapeType Write SetShapeType Default stRectangle;
    Property RoundRHRadius : Double Read FRHR Write SetRHR;
    Property RoundRVRadius : Double Read FRVR Write SetRVR;
  end;

  { TLabelObject }

  TLabelObject = Class(TCoverObject)
  private
    FAlignMent: TAlignMent;
    FCaption: String;
    FFillColor: TColor;
    FFont: TFont;
    FLayout: TTextLayout;
    FTransParent: Boolean;
    FWordWrap: Boolean;
    procedure SetAlignment(const AValue: TAlignMent);
    procedure SetCaption(const AValue: String);
    procedure SetFillColor(const AValue: TColor);
    procedure SetFont(const AValue: TFont);
    procedure Setlayout(const AValue: TTextLayout);
    procedure SetTransParent(const AValue: Boolean);
    procedure SetWordWrap(const AValue: Boolean);
  Protected
    Class Function GetMementoClass : TMementoClass; override;
    Procedure DoApplyMemento(AMemento : TMemento); override;
    Procedure DoFillMemento(AMemento : TMemento); override;
    procedure DrawObject(ACanvas : TCanvas; PageRect : TRect; ADPI : Integer); override;
  Public
    Constructor Create(AOwner : TComponent); override;
    Destructor Destroy; override;
  Published
    Property Caption : String Read FCaption Write SetCaption;
    Property Font : TFont Read FFont Write SetFont;
    Property Alignment : TAlignMent Read FAlignMent Write SetAlignment;
    Property Layout : TTextLayout Read FLayout Write Setlayout;
    Property Transparent : Boolean Read FTransParent Write SetTransParent;
    Property FillColor : TColor Read FFillColor Write SetFillColor default clWhite;
    Property WordWrap : Boolean Read FWordWrap Write SetWordWrap;
  end;

  { TImageObject }

  TImageObject = Class(TCoverObject)
  private
    FCentered: Boolean;
    FPicture: TPicture;
    FStretched: Boolean;
    procedure SetCentered(const AValue: Boolean);
    procedure SetPicture(const AValue: TPicture);
    procedure SetStretched(const AValue: Boolean);
  Protected
    Class Function GetMementoClass : TMementoClass; override;
    Procedure DoApplyMemento(AMemento : TMemento); override;
    Procedure DoFillMemento(AMemento : TMemento); override;
    procedure DrawObject(ACanvas : TCanvas; PageRect : TRect; ADPI : Integer); override;
  Public
    Constructor Create(AOwner : TComponent); override;
    Destructor Destroy; override;
  Published
    Property Picture : TPicture Read FPicture Write SetPicture;
    Property Stretched : Boolean Read FStretched Write SetStretched;
    Property Centered : Boolean Read FCentered Write SetCentered;
  end;

  // Alignment actions
  THAlignAction = (haNone,haLeft,haCenter,haRight,haSpace,haCentp);
  TVAlignAction = (vaNone,vaTop,vaCenter,vaBottom,vaSpace,vaCentp);
  TSizeAdjust = (saNone,saLargest,saSmallest,saValue);

  { TSelectionList }

  TSelectionList = Class(TFPList)
  private
    FHMax: Double;
    FHMin: Double;
    FPage: TCoverPage;
    FPHCenter: Double;
    FPVCenter: Double;
    FVMax: Double;
    FVMin: Double;
    procedure AlignControl(Index: Integer; Hor: THAlignAction;
      Ver: TValignAction);
    function GetC(Index : Integer): TCoverObject;
    function GetHCenter: Double;
    function GetVCenter: Double;
    procedure SetPage(const AValue: TCoverPage);
  Public
    Procedure CalcAlignRect;
    Procedure AlignObjects (Hor : THAlignAction; Ver : TValignAction);
    Procedure AdjustSizes(Height : TSizeAdjust; HSize : Double;
                          Width : TSizeAdjust; WSize: Double);
    procedure DeleteSelection;
    Property Page : TCoverPage Read FPage Write SetPage;
    Property CoverObjects[Index : Integer] : TCoverObject Read GetC;default;
    Property HMin : Double Read FHMin;
    Property HMax : Double Read FHMax;
    Property VMin : Double Read FVMin;
    Property VMax : Double Read FVMax;
    Property HCenter : Double Read GetHCenter;
    Property VCenter : Double Read GetVCenter;
    Property PageHCenter : Double Read FPVCenter;
    Property PageVCenter : Double Read FPHCenter;
  end;

  TSelection = Class(TComponent)
  private
    FSelection: TSelectionlist;
  Protected
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
  Public
    Property Selection : TSelectionlist Read FSelection Write FSelection;
  end;

  { Mementos }

  TCoverMemento = Class(TMemento)
  Protected
    FDims : Array[1..4] of Double;
  end;

  TShapeMemento = Class(TCoverMemento)
  Protected
    FPenColor: TColor;
    FFillColor: TColor;
    FPenWidth: Integer;
    FRHR: Double;
    FRVR: Double;
    FShapeType: TShapeType;
    FTransParent: Boolean;
  end;

  { TLabelMemento }

  TLabelMemento = Class(TCoverMemento)
  Protected
    FAlignMent: TAlignMent;
    FCaption: String;
    FFillColor: TColor;
    FFont: TFont;
    FLayout: TTextLayout;
    FTransParent: Boolean;
    FWordWrap: Boolean;
  Public
    Constructor CreateMemento; override;
    Destructor Destroy; Override;
  end;

  { TTrackListMemento }

  TTrackListMemento = Class(TCoverMemento)
  Protected
    FDisc: TCDDisc;
    FFont: TFont;
    FTrackElements : Array[0..3] of TTrackElement;
  Public
    Constructor CreateMemento; override;
    Destructor Destroy; Override;
  end;

  { TImageMemento }

  TImageMemento = Class(TCoverMemento)
  Protected
    FCentered: Boolean;
    FPicture: TPicture;
    FStretched: Boolean;
  Public
    Constructor CreateMemento; override;
    Destructor Destroy; Override;
  end;

  TCoverPageMemento = Class(TMemento)
  Protected
    FHeight: Double;
    FWidth: Double;
    FDepth: Double;
    FRadius1: Double;
    FRadius2: Double;
  end;

  { TCDCoverAction }

  TCDCoverAction = Class(TObject)
  Public
    Procedure Undo; virtual; abstract;
    Function DisplayName : String; virtual abstract;
    Function Description : String; virtual;
  end;

  { TCDCoverActions }

  TCDCoverActions = Class(TObject)
  Private
    FActions : TObjectList;
    FMaxCount: Integer;
    function GetA(Index : Integer): TCDCoverAction;
    function GetActionCount: Integer;
    procedure SetMaxCount(const AValue: Integer);
  Public
    Constructor Create(AMaxCount : Integer);
    Destructor Destroy; override;
    Function UndoLastAction : Boolean;
    Function LastAction : TCDCoverAction;
    Function AddAction(AAction : TCDCoverAction) : Integer;
    Property Actions[Index : Integer] : TCDCoverAction Read GetA; default;
    Property ActionCount : Integer Read GetActionCount;
    Property MaxCount : Integer Read FMaxCount Write SetMaxCount;
  end;


  { TSingleObjectAction }

  TSingleObjectAction = Class(TCDCoverAction)
  private
    FObject: TCoverObject;
  Public
    Constructor Create(AObject : TCoverObject); virtual;
    Property CoverObject : TCoverObject Read FObject;
  end;

  { TSelectedObjectsAction }

  TSelectedObjectsAction = Class(TCDCoverAction)
  private
    FObjects: TSelectionList;
  Public
    Constructor Create(ASelection : TSelectionlist); virtual;
    Destructor Destroy; override;
    Property Objects : TSelectionList Read FObjects;
  end;

  { TAddObjectAction }

  TAddObjectAction = Class(TSingleObjectAction)
  Public
    Procedure Undo; override;
    Function DisplayName : String; override;
  end;

  { TEditObjectAction }

  TEditObjectAction = Class(TSingleObjectAction)
  private
    FMemento : TMemento;
  Public
    Constructor Create(AObject : TCoverObject); override;
    Destructor Destroy; override;
    Procedure Undo; override;
    Function DisplayName : String; override;
  end;

  { TResizeObjectAction }

  TResizeObjectAction = Class(TSingleObjectAction)
  private
    FDims : Array[1..4] of Double;
  Public
    Constructor Create(AObject : TCoverObject); override;
    Procedure Undo; override;
    Function DisplayName : String; override;
  end;

  { TDeleteObjectsAction }

  TDeleteObjectsAction = Class(TSelectedObjectsAction)
  private
    FParent : TCoverPage;
  Public
    Constructor Create(ASelection : TSelectionList); override;
    Destructor Destroy; override;
    Procedure Undo; override;
    Function DisplayName : String; override;
  end;

  TObjectOrigin = Record
    X,Y : Double;
  end;
  PObjectOrigin = ^TObjectOrigin;

  { TMoveObjectsAction }

  TMoveObjectsAction = Class(TSelectedObjectsAction)
  private
    FOrigins : PObjectOrigin;
  Public
    Constructor Create(ASelection : TSelectionList); override;
    Destructor Destroy; override;
    Procedure Undo; override;
    Function DisplayName : String; override;
  end;

  { TAlignObjectsAction }

  TAlignObjectsAction = Class(TMoveObjectsAction)
  Public
    Function DisplayName : String; override;
  end;

  TObjectDimensions = Array[1..4] of Double;
  PObjectDimensions = ^TObjectDimensions;

  { TResizeObjectsAction }

  TResizeObjectsAction = Class(TSelectedObjectsAction)
  private
    FDimensions : PObjectDimensions;
  Public
    Constructor Create(ASelection : TSelectionList); override;
    Destructor Destroy; override;
    Procedure Undo; override;
    Function DisplayName : String; override;
  end;

Function ToPoints(AUnits : Double; ADPI : Integer) : Integer;
Function ToUnits(Points : Integer; ADPI : Integer) : Double;

Function PageTypeName(APageType : TPageType) : String;
Function ElementTypeName(AElementType : TElementType) : String;
Function CreateUniqueName(AOwner : TComponent; Const BaseName : String) : String;
Function CreateUniqueName(AComponent : TComponent; AOwner : TComponent = Nil) : String;

implementation

Resourcestring
  SBookletFront = 'Booklet (front)';
  SBookletBack  = 'Booklet (back)';
  SInlay        = 'Inlay';
  SDiscLabel    = 'Disc label';
  SerrInvalidPosition = 'Invalid position : %d';

  STrackNo            = 'Track nr.';
  STrackPerformer     = 'Performer';
  STrackTitle         = 'Title';
  STrackDuration      = 'Duration';

  SErrMementoNotApplicable = 'Memento class %s is not applicable to an object of class %s';

Function CreateUniqueName(AOwner : TComponent; Const BaseName : String) : String;

Var
  I : Integer;


begin
  If (AOwner=Nil) then
    Result:=BaseName+'1'
  else
    begin
    I:=0;
    Repeat
      Inc(I);
      Result:=BaseName+IntToStr(I);
    Until AOwner.FindComponent(Result)=Nil;
    end;
end;

Function CreateUniqueName(AComponent : TComponent; AOwner : TComponent = Nil) : String;

begin
  If (AComponent=Nil) then
    Exit;
  If (AOwner=Nil) then
    AOwner:=AComponent.Owner;
  Result:=CreateUniqueName(AOwner,AComponent.ClassName);
end;


Function CreateDefaultFont : TFont;

begin
  Result:=TFont.Create;
  Result.Name:='Courier New';
  Result.Size:=10;
  Result.Color:=clBlack;
end;
Function PageTypeName(APageType : TPageType) : String;

begin
  Case APageType of
    ptBookletFront : Result:=SBookletFront;
    ptBookletBack  : Result:=SBookletBack;
    ptInlay        : Result:=SInlay;
    ptDiscLabel    : Result:=SDiscLabel;
  end;
end;

function ElementTypeName(AElementType: TElementType): String;
begin
  Case AElementType of
    etTrackNo   : Result:=STrackNo;
    etPerformer : Result:=STrackPerformer;
    etTitle     : Result:=STrackTitle;
    etDuration  : Result:=STrackDuration;
  end;

end;

Const
  AnInch = 2.54;

Function ToPoints(AUnits : Double; ADPI : Integer) : Integer;

begin
  Result:=Round(AUnits*(ADPI/AnInch));
end;

function ToUnits(Points: Integer; ADPI: Integer): Double;
begin
  Result:=((Points * AnInch)/ADPI);
end;

{ TCDDisc }

procedure TCDDisc.SetPerformer(const AValue: String);
begin
  if FPerformer=AValue then exit;
  FPerformer:=AValue;
  Changed;
end;

function TCDDisc.GetCaption: String;

begin
  Result:=FTitle;
  If (FPerformer<>'') then
    begin
    If (Result<>'') then
      Result:=' - '+Result;
    Result:=FPerformer+Result;
    end;
end;

procedure TCDDisc.SetComment(const AValue: String);
begin
  if FComment=AValue then exit;
  FComment:=AValue;
  Changed;
end;

procedure TCDDisc.SetTitle(const AValue: String);
begin
  if FTitle=AValue then exit;
  FTitle:=AValue;
  Changed;
end;

procedure TCDDisc.SetTracks(const AValue: TCDCoverTracks);
begin
  if FTracks=AValue then exit;
  FTracks.Assign(AValue);
end;

procedure TCDDisc.SetYear(const AValue: Integer);
begin
  if FYear=AValue then exit;
  FYear:=AValue;
  Changed;
end;


constructor TCDDisc.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FTracks:=TCDCoverTracks.Create(TCDCoverTrack);
end;

destructor TCDDisc.Destroy;
begin
  FreeAndNil(FTracks);
  inherited Destroy;
end;

{ TCDCover }

function TCDCover.GetDisc(AIndex : Integer): TCDDisc;
begin
  Result:=TCDDisc(FDiscs[AIndex]);
end;

function TCDCover.GetDiscCount: Integer;
begin
  Result:=FDiscs.Count;
end;

function TCDCover.GetPage(AIndex : Integer): TCoverPage;
begin
  Result:=TCoverPage(FPages[AIndex]);
end;

function TCDCover.GetPageCount: Integer;
begin
  Result:=FPages.Count;
end;

function TCDCover.GetPerformer: String;
begin
  Result:=FPerformer;
  If (Result='') and (DiscCount>0) then
    Result:=Disc[0].Performer;
end;

function TCDCover.GetTitle: String;
begin
  Result:=FTitle;
  If (Result='') and (DiscCount>0) then
    Result:=Disc[0].Title;
end;

procedure TCDCover.SetDescription(const AValue: TStrings);
begin
  if FDescription=AValue then exit;
  FDescription.Assign(AValue);
  Changed;
end;

procedure TCDCover.SetPerformer(const AValue: String);
begin
  if FPerformer=AValue then exit;
  FPerformer:=AValue;
  Changed;
end;

procedure TCDCover.SetTitle(const AValue: String);
begin
  if FTitle=AValue then exit;
  FTitle:=AValue;
  Changed;
end;

procedure TCDCover.GetChildren(Proc: TGetChildProc; Root: TComponent);

Var
  I : integer;

begin
  inherited GetChildren(Proc, Root);
  For I:=0 to DiscCount-1 do
    Proc(Disc[i]);
  For I:=0 to PageCount-1 do
    Proc(Page[i]);
end;

constructor TCDCover.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDiscs:=TFPlist.Create;
  FPages:=TFPlist.Create;
  FDescription:=TStringList.Create;
end;

Procedure TCDCover.Clear;

var
  I : Integer;

begin
  For I:=FDiscs.Count-1 downto 0 do
    TObject(FDiscs[i]).free;
  For I:=FPages.Count-1 downto 0 do
    TObject(FPages[i]).free;
end;

procedure TCDCover.LoadFromStream(Stream: TStream);

Var
  M : TMemoryStream;
  I : Integer;

begin
  M:=TMemoryStream.Create;
  try
    ObjectTextToBinary(Stream,M);
    M.Position:=0;
    M.ReadComponent(Self);
    For I:=0 to ComponentCount-1 do
      If (Components[i].Name='') then
        Components[i].Name:=CreateUniqueName(Components[i],Self);
  finally
    M.Free;
  end;
end;

procedure TCDCover.SaveToStream(Stream: TStream);

Var
  M : TMemoryStream;

begin
  M:=TMemoryStream.Create;
  try
    M.WriteComponent(Self);
    M.Position:=0;
    ObjectBinaryToText(M,Stream);
  finally
    M.Free;
  end;
end;

procedure TCDCover.LoadFromFile(const AFileName: String);
Var
  F : TFileStream;
begin
  F:=TFileStream.Create(AFileName,fmopenRead);
  try
    LoadFromStream(F);
  finally
    F.free;
  end;
end;

procedure TCDCover.SaveToFile(const AFileName: String);

Var
  F : TFileStream;

begin
  F:=TFileStream.Create(AFileName,fmCreate);
  try
    SaveToStream(F);
  finally
    F.free;
  end;
end;

procedure TCDCover.GetDiscs(List: TList);

Var
  I : Integer;

begin
  For I:=0 to FDiscs.Count-1 do
    List.Add(FDiscs[i]);
end;

procedure TCDCover.GetPages(List: TList);
Var
  I : Integer;

begin
  For I:=0 to FPages.Count-1 do
    List.Add(FPages[i]);
end;

destructor TCDCover.Destroy;
begin
  Clear;
  FreeAndNil(FDiscs);
  FreeAndNil(FPages);
  inherited Destroy;
end;

procedure TCDCover.Notification(AComponent: TComponent; Operation: TOperation);

Var
  L : TFPList;

begin
  inherited Notification(AComponent, Operation);
  If AComponent is TCoverPage then
    L:=FPages
  else If AComponent is TCDDisc then
    L:=FDiscs
  else
    L:=Nil;
  If (L<>Nil) then
    if (Operation=opInsert) then
      L.Add(AComponent)
    else
      L.Remove(AComponent);
end;

function TCDCover.AddDisc: TCDDisc;

  Function Discname : string;

  Var
    N : Integer;

  begin
    N:=0;
    Repeat
      Inc(N);
      Result:='Disc'+IntToStr(N);
    Until FindComponent(Result)=Nil;
  end;


begin
  Result:=TCDDisc.Create(Self);
  Result.Name:=DiscName;
end;

function TCDCover.AddPage(APageType: TPageType): TCoverPage;

  Function PageName : string;

  Var
    N : Integer;

  begin
    N:=0;
    Repeat
      Inc(N);
      Result:='Page'+IntToStr(N);
    Until FindComponent(Result)=Nil;
  end;

begin
  Case APageType of
    ptBookletFront : Result:=TBookletFrontPage.Create(Self);
    ptBookletBack : Result:=TBookletBackPage.Create(Self);
    ptInlay : Result:=TInlayPage.Create(Self);
    ptDiscLabel : Result:=TDiscLabelPage.Create(Self);
  end;
  Result.Name:=PageName;
end;

function TCDCover.HavePageType(APageType: TPageType): Boolean;

Var
  I : Integer;

begin
  Result:=False;
  I:=PageCount-1;
  While (Not Result) and (I>=0) do
    begin
    Result:=(Page[i].PageType=APageType);
    Dec(I);
    end;
end;


{ TCoverPage }


procedure TCoverPage.GetChildren(Proc: TGetChildProc; Root: TComponent);

Var
  I : Integer;

begin
  inherited GetChildren(Proc, Root);
  For I:=0 to ObjectCount-1 do
    Proc(Objects[i]);
end;

constructor TCoverPage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FObjects:=TFPList.Create;
  FWidth:=12;
  FHeight:=12;
end;

destructor TCoverPage.Destroy;
begin
  FreeAndNil(FObjects);
  inherited Destroy;
end;

procedure TCoverPage.Clear;

var
  I : integer;

begin
  For I:=FObjects.Count-1 downto 0 do
    TObject(FObjects[i]).Free;
end;

procedure TCoverPage.DrawObject(AObject: TCoverObject; ACanvas: TCanvas;
  Origin: TPoint; ADPI: Integer);
Var
  R : TRect;
  I : Integer;

begin
  R.TopLeft:=Origin;
  R.Right:=R.Left+ToPoints(Width,ADPI);
  R.Bottom:=R.Top+ToPoints(Height,ADPI);
  AObject.DrawObject(ACanvas,R,ADPI);
end;


procedure TCoverPage.DrawBookletFront(ACanvas: TCanvas; PageRect : TRect; ADPI: Integer);

begin
  // Nothing to do
end;

procedure TCoverPage.DrawBookletBack(ACanvas: TCanvas; PageRect : TRect; ADPI: Integer);

begin
  // Nothing to do
end;

procedure TCoverPage.DrawInlay(ACanvas: TCanvas; PageRect : TRect; ADPI: Integer);

begin
  // Nothing to do
end;


procedure TCoverPage.DrawDiscLabel(ACanvas: TCanvas; PageRect : TRect; ADPI: Integer);

begin
end;

procedure TCoverPage.DrawPage(ACanvas: TCanvas; Origin: TPoint; ADPI: Integer);

Var
  R : TRect;
  I : Integer;

begin
  R.TopLeft:=Origin;
  R.Right:=R.Left+ToPoints(Width,ADPI);
  R.Bottom:=R.Top+ToPoints(Height,ADPI);
  ACanvas.Brush.Color:=clWhite;
  ACanvas.Brush.Style:=bsSolid;
  ACanvas.FillRect(R);
  ACanvas.Pen.Color:=clBlack;
  ACanvas.Pen.Width:=1;
  ACanvas.Rectangle(R);
  Case GetPageType of
    ptBookletFront : DrawBookletFront(ACanvas,R,ADPI);
    ptBookletBack  : DrawBookletBack(ACanvas,R,ADPI);
    ptInlay        : DrawInlay(ACanvas,R,ADPI);
    ptDiscLabel    : DrawDiscLabel(ACanvas,R,ADPI);
  end;
  For I:=0 to ObjectCount-1 do
    begin
    ACanvas.ClipRect:=Objects[i].GetObjectRect(ACanvas,Origin,ADPI);
    ACanvas.Clipping:=True;
    Objects[I].DrawObject(ACanvas,R,ADPI);
    end;
  ACanvas.Clipping:=False;
end;

procedure TCoverPage.Notification(AComponent: TComponent; Operation: TOperation
  );
begin
  inherited Notification(AComponent, Operation);
  If AComponent is TCoverObject then
    begin
    If Operation=opRemove then
      FObjects.Remove(AComponent);
    If not (csDestroying in ComponentState)  then
      Changed;
    end;
end;

function TCoverPage.GetObjectAt(X, Y: Double): TCoverObject;

Var
  I : Integer;

begin
  I:=ObjectCount-1;
  Result:=Nil;
  While (Result=Nil) and (I>=0) do
    begin
    Result:=Objects[i];
    If Not Result.HavePoint(X,Y) then
      Result:=Nil;
    Dec(I);
    end;
end;

procedure TCoverPage.SetHeight(const AValue: Double);
begin
  if FHeight=AValue then exit;
  FHeight:=AValue;
  Changed;
end;


function TCoverPage.GetObject(AIndex : Integer): TCoverObject;
begin
  Result:=TCoverObject(FObjects[AIndex]);
end;

function TCoverPage.GetObjectCount: Integer;
begin
  Result:=FObjects.Count;
end;


procedure TCoverPage.SetWidth(const AValue: Double);
begin
  if FWidth=AValue then exit;
  FWidth:=AValue;
  Changed;
end;

class function TCoverPage.GetMementoClass: TMementoClass;
begin
  Result:=TCoverPageMemento;
end;

procedure TCoverPage.DoApplyMemento(AMemento: TMemento);

Var
  M : TCoverPageMemento;

begin
  M:=TCoverPageMemento(AMemento);
  M.FWidth:=FWidth;
  M.FHeight:=FHeight;
  If self is TInlayPage then
    M.FDepth:=TInlayPage(Self).FDepth
  else if self is TDiscLabelPage then
    begin
    M.FRadius1:=TDiscLabelPage(Self).FRadius1;
    M.FRadius2:=TDiscLabelPage(Self).FRadius2;
    end;
end;

procedure TCoverPage.DoFillMemento(AMemento: TMemento);
Var
  M : TCoverPageMemento;

begin
  M:=TCoverPageMemento(AMemento);
  FWidth  := M.FWidth;
  FHeight := M.FHeight;
  If self is TInlayPage then
    M.FDepth:=TInlayPage(Self).FDepth
  else if self is TDiscLabelPage then
    begin
    TDiscLabelPage(Self).FRadius1 := M.FRadius1;
    TDiscLabelPage(Self).FRadius2 := M.FRadius2;
    end;
end;

{ TCDCoverTrack }

function TCDCoverTrack.GetPerformer: String;
begin
  Result:=FPerformer;
  If (Result='') and Assigned(Collection) and (Collection is TCDCoverTracks) then
    If Assigned(TCDCoverTracks(Collection).Disc) then
      Result:=TCDCoverTracks(Collection).Disc.Performer;
end;

function TCDCoverTrack.GetCaption: String;
begin
  Result:=FTitle;
  If (FPerformer<>'') then
    begin
    If (Result<>'') then
      Result:=' - '+Result;
    Result:=FPerformer+Result;
    end;
end;


procedure TCDCoverTrack.SetDuration(const AValue: TDateTime);
begin
  If FDuration=AValue then Exit;
  FDuration:=AValue;
  Changed;
end;

procedure TCDCoverTrack.SetExtra(const AValue: String);
begin
  If (FExtra=AValue) then Exit;
  FExtra:=Avalue;
  Changed;
end;

procedure TCDCoverTrack.SetPerformer(const AValue: String);
begin
  If (FPerformer=AValue) then Exit;
  FPerformer:=Avalue;
  Changed;
end;

procedure TCDCoverTrack.SetTitle(const AValue: String);
begin
  If (FTitle=AValue) then Exit;
  FTitle:=Avalue;
  Changed;
end;

procedure TCDCoverTrack.SetYear(const AValue: Integer);
begin
  if FYear=AValue then exit;
  FYear:=AValue;
  Changed;
end;

procedure TCDCoverTrack.Changed;
begin
{$ifdef usedelegation}
  ObservedHook.NotifyObservers(Self,ooChanged);
{$else}
  NotifyObservers(Self,ooChanged);
{$endif}
end;

procedure TCDCoverTrack.Assign(Source: TPersistent);

Var
  T : TCDCoverTrack;
  TR : TCDTrack;
begin
  if (Source is TCDCoverTrack) then
    begin
    T:=Source as TCDCoverTrack;
    FTitle:=T.FTitle;
    FExtra:=T.FExtra;
    FPerformer:=T.FPerformer;
    FDuration:=T.FDuration;
    Changed;
    end
  else if (Source is TCDTrack) then
    begin
    TR:=Source as TCDTrack;
    FTitle:=TR.Title;
    FExtra:=TR.Extra;
    FPerformer:=TR.Performer;
    FDuration:=TR.Duration;
    Changed;
    end
  else
    inherited Assign(Source);
end;

{ TCDCoverTracks }

function TCDCoverTracks.GetT(AIndex: Integer): TCDCoverTrack;
begin
  Result:=TCDCoverTrack(Items[AIndex]);
end;

procedure TCDCoverTracks.SetT(AIndex: Integer; const AValue: TCDCoverTrack);
begin
  Items[AIndex]:=AValue;
end;


function TCDCoverTracks.AddTrack(const ATitle, AExtra: String;
  ADuration: TDateTime): TCDCoverTrack;
begin
  Result:=Add As TCDCoverTrack;
  Result.FTitle:=Atitle;
  Result.FExtra:=AExtra;
  Result.FDuration:=ADuration;
  Result.Changed;
end;

function TCDCoverTracks.AddTrack(const ATitle, AExtra: String): TCDCoverTrack;
begin
  Result:=AddTrack(ATitle,AExtra,0);
end;

function TCDCoverTracks.AddTrack(const ATitle: String): TCDCoverTrack;
begin
  Result:=AddTrack(ATitle,'',0);
end;

{ TCoverObject }

function TCoverObject.GetDim(AIndex: integer): Double;
begin
  Result:=FDims[Aindex]
end;

procedure TCoverObject.SetDim(AIndex: integer; const AValue: Double);
begin
  FDims[Aindex]:=AValue;
  Changed;
end;

class function TCoverObject.GetMementoClass: TMementoClass;
begin
  Result:=TCoverMemento;
end;

procedure TCoverObject.DoApplyMemento(AMemento: TMemento);
begin
  FDims:=TCoverMemento(AMemento).FDims;
end;

procedure TCoverObject.DoFillMemento(AMemento: TMemento);
begin
  TCoverMemento(AMemento).FDims:=FDims;
end;

function TCoverObject.GetObjectRect(ACanvas: TCanvas; Const PageRect: TRect;
  ADPI: Integer): TRect;

begin
  Result:=GetObjectRect(ACanvas,PageRect.TopLeft,ADPI);
end;

function TCoverObject.GetObjectRect(ACanvas: TCanvas; Const PageOrigin: TPoint;
  ADPI: Integer): TRect;
begin
  Result.Top:=PageOrigin.Y+ToPoints(Top,ADPI);
  Result.Left:=PageOrigin.X+ToPoints(Left,ADPI);
  Result.Bottom:=Result.Top+ToPoints(Height,ADPI);
  Result.Right:=Result.Left+ToPoints(Width,ADPI);
end;

constructor TCoverObject.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDims[1]:=2;
  FDims[2]:=1;
end;

function TCoverObject.HavePoint(X, Y: Double): Boolean;
begin
  Result:=(X>=Left) and (Y>=Top);
  Result:=Result and (X<=Left+Width) and (Y<=Top+Height);
end;

procedure TCoverObject.SetParentComponent(Value: TComponent);
begin
  If Value is TCoverPage then
    begin
    FParent:=Value;
    TCoverPage(Value).FObjects.Add(Self);
    end;
end;

function TCoverObject.GetParentComponent: TComponent;
begin
  Result:=FParent;
end;

{ TTrackelement }

procedure TTrackelement.SetFont(const AValue: TFont);
begin
  If (AValue=FFont) then exit;
  FFont.Assign(AValue);
  ParentFont:=False;
  Changed;
end;

procedure TTrackelement.SetAlignMent(const AValue: TAlignment);
begin
  if FAlignment=AValue then exit;
  FAlignment:=AValue;
end;

procedure TTrackelement.SetFormat(const AValue: String);
begin
  if FFormat=AValue then exit;
  FFormat:=AValue;
  Changed;
end;

procedure TTrackelement.SetParentFont(const AValue: Boolean);
begin
  if FParentFont=AValue then exit;
  FParentFont:=AValue;
  ParentFontChanged;
  Changed;
end;

procedure TTrackelement.SetPosition(const AValue: Integer);
begin
  if FPosition=AValue then exit;
  If (AValue<1) and (AValue>4) then
    Raise Exception.CreateFmt(SerrInvalidPosition,[AValue]);
  FTrackList.Reposition(Self,AValue);
end;

procedure TTrackelement.ParentFontChanged;

begin
  if FParentFont then
    begin
    FOnt:=FTrackList.Font;
    FParentFont := true;
    end;
end;

function TTrackelement.GetFontStored: boolean;
begin
  Result:=Not FParentFont;
end;

function TTrackelement.GetElementName: String;
begin
end;

procedure TTrackelement.SetVisible(const AValue: Boolean);
begin
  if FVisible=AValue then exit;
  FVisible:=AValue;
  Changed;
end;

procedure TTrackelement.SetWidth(const AValue: Double);
begin
  if FWidth=AValue then exit;
  FWidth:=AValue;
end;

procedure TTrackelement.FontChanged(Sender: TObject);
begin
  ParentFont := False;
end;

constructor TTrackelement.Create;
begin
  inherited Create;
  FFont:=CreateDefaultFont;
  FFont.OnChange := @FontChanged;
end;

destructor TTrackelement.Destroy;
begin
  FreeAndNil(FFont);
  inherited Destroy;
end;

procedure TTrackelement.Assign(Source: TPersistent);

Var
  E : TTrackelement;

begin
  If Source is TTrackElement then
    begin
    E:=Source as TTrackElement;
    FAlignment:=E.FAlignment;
    FFormat:=E.FFormat;
    FParentFont:=E.FParentFont;
    FPosition:=E.FPosition;
    FVisible:=E.FVisible;
    FTrackList:=E.FTrackList;
    FFont.Assign(E.FFont);
    FWidth:=E.Width;
    FElementType:=E.FElementType;
    end
  else
    inherited Assign(Source);
end;

procedure TTrackelement.Changed;
begin
  Inherited;
  If Assigned(FTrackList) then
    FTrackList.Changed;
end;

{ TTrackList }

function TTrackList.GetTrackElement(AIndex: integer): TTRackElement;
begin
  Result:=FTrackElements[Aindex];
end;

procedure TTrackList.SetDisc(const AValue: TCDDisc);
begin
  if FDisc=AValue then exit;
  If Assigned(FDisc) then
    FDisc.RemoveFreeNotification(Self);
  FDisc:=AValue;
  If Assigned(FDisc) then
    FDisc.FreeNotification(Self);
  Changed;

end;

procedure TTrackList.SetFont(const AValue: TFont);
begin
  If AValue=FFOnt then exit;
  FFont.Assign(AValue);
  Changed;
end;

procedure TTrackList.SetTrackElement(AIndex: integer;
  const AValue: TTRackElement);
begin
  FTrackElements[Aindex].Assign(AValue);
end;

class function TTrackList.GetMementoClass: TMementoClass;
begin
  Result:=TTrackListMemento;
end;

procedure TTrackList.DoApplyMemento(AMemento: TMemento);

Var
  M : TTrackListMemento;
  i : Integer;

begin
  inherited DoApplyMemento(AMemento);
  M:=TTrackListMemento(Amemento);
  FDisc:=M.FDisc;
  FFont.Assign(M.FFont);
  For I:=0 to 3 do
    FTrackElements[i].Assign(M.FTrackElements[i]);
end;

procedure TTrackList.DoFillMemento(AMemento: TMemento);
Var
  M : TTrackListMemento;
  i : Integer;
begin
  inherited DoFillMemento(AMemento);
  M:=TTrackListMemento(Amemento);
  M.FDisc:=FDisc;
  M.FFont.Assign(FFont);
  For I:=0 to 3 do
    M.FTrackElements[i].Assign(FTrackElements[i]);
end;

procedure TTrackList.Reposition(FElement: TTrackElement; NewPos : Integer);

Var
  I,p : Integer;

begin
  If Not (csLoading in Componentstate) then
    begin
    P:=FElement.Position;
    If NewPos<FElement.FPosition then
      For I:=0 to 3 do
        begin
        P:=FTrackElements[i].Position;
        If (P>=NewPos) and (P<FElement.FPosition) then
          Inc(FTrackElements[i].FPosition);
        end;
    If NewPos>FElement.FPosition then
      For I:=0 to 3 do
        begin
        P:=FTrackElements[i].Position;
        If (P<=NewPos) and (P>FElement.FPosition) then
          Dec(FTrackElements[i].FPosition);
        end;
    end;
  FElement.FPosition:=NewPos;
  FElement.Changed;
end;

procedure TTrackList.DrawObject(ACanvas: TCanvas; PageRect: TRect; ADPI: Integer
  );

Var
  R,RL : TRect;
  I,J,X,Y,H,W,MH : Integer;
  T : TCDCoverTrack;
  E : TTrackElement;
  S : String;
  TT : TTextStyle;
  A : Array[0..3] of TTrackElement;

begin
  R:=GetObjectRect(ACanvas,PageRect,ADPI);
  FillChar(TT,SizeOf(TTextStyle),0);
  ACanvas.Pen.Color:=clBlack;
  ACanvas.Brush.Color:=clWhite;
  ACanvas.FillRect(R);
  If Not Assigned(FDisc) or (FDisc.Tracks.Count=0) then
    Exit;
  Y:=R.Top;
  // Order elements
  For I:=0 to 3 do
    A[FTrackelements[i].FPosition-1]:=FTrackelements[i];
  For I:=0 to FDisc.Tracks.Count-1 do
    begin
    X:=R.Left;
    T:=FDisc.Tracks[I];
    MH:=0;
//    If (Y+MH<=R.Bottom) then
      For J:=0 to 3 do
        begin
        E:=A[J];
        If E.Visible then
          begin
          S:=GetElementText(E,T);
          ACanvas.Font:=E.Font;
          H:=ACanvas.TextHeight(S);
          RL.Left:=X;
          RL.Top:=Y;
          RL.Bottom:=RL.Top+H;
          W:=ToPoints(E.Width,ADPI);
          // If zero, add next right behind this one.
          If (W<=0) then
            W:=ACanvas.TextWidth(S);
          RL.Right:=RL.Left+W;
          If H>MH then
            MH:=H;
          TT.Alignment:=E.Alignment;
          ACanvas.TextRect(RL,RL.Left,RL.Top,S,TT);
          Inc(X,W+2);
          end;
        end;
    Y:=Y+MH+2;
    end;
end;

procedure TTrackList.Loaded;

Var
  I : Integer;

begin
  inherited Loaded;
  For I:=0 to 3 do
    begin
    If FTrackElements[i].ParentFont then
      begin
      FTrackElements[i].Font.Assign(Font);
      FTrackElements[i].ParentFont:=True;
      end;
    end;
end;

Function TTrackList.GetElementText(E : TTrackElement; T : TCDCoverTrack) : String;

begin
  Case E.ElementType of
    etTrackNo   : Result:=IntToStr(T.Index+1);
    etPerformer : Result:=T.Performer;
    etTitle     : Result:=T.Title;
    etDuration  : If (T.Duration<>0) then
                    Result:=FormatDateTime('nn:ss',T.Duration)
                  else
                    Result:=''
  end;
end;

function TTrackList.GetT(Index : Integer): TTrackElement;
begin
  Result:=FTrackElements[Index];
end;

constructor TTrackList.Create(Aowner: TComponent);

Const
  StdWidths : Array[TElementType] of double
             = (0.7,2,8,1.2);

Var
  I : Integer;
  W : Double;
  e : TElementType;
  TE : TTrackElement;
begin
  inherited Create(AOwner);
  FFont:=CreateDefaultFont;
  W:=0;
  for e:=Low(TelementType) to High(TElementType) do
    begin
    I:=Ord(e);
    TE:=TTrackElement.Create;
    TE.FtrackList:=Self;
    TE.FWidth:=StdWidths[e];
    TE.FElementType:=E;
    TE.FVisible:=E<>etPerformer;
    TE.FPosition:=I+1;
    FTrackElements[i]:=TE;
    If TE.FVisible then
      W:=W+FTrackElements[i].FWidth;
    end;
  // By default, don't show performer
  Width:=W;
  If (AOwner is TCDCover) then
    If (TCDCover(AOwner).DiscCount>0) then
      Disc:=TCDCover(AOwner).Disc[0];
end;

destructor TTrackList.Destroy;

Var
  I : Integer;

begin
  FreeAndNil(FFont);
  for I:=0 to 3 do
    FreeAndNil(FTrackElements[i]);
  inherited Destroy;
end;

procedure TTrackList.Notification(AComponent: TComponent; Operation: TOperation
  );
begin
  If (Operation=opRemove) and (AComponent=FDisc) then
    FDisc:=Nil;
  inherited Notification(AComponent, Operation);
end;

{ TShapeObject }

procedure TShapeObject.SetFillColor(const AValue: TColor);
begin
  if FFillColor=AValue then exit;
  FFillColor:=AValue;
  Changed;
end;

procedure TShapeObject.SetPenColor(const AValue: TColor);
begin
  if FPenColor=AValue then exit;
  FpenColor:=AValue;
  Changed;
end;

procedure TShapeObject.SetpenWidth(const AValue: Integer);
begin
  if FPenWidth=AValue then exit;
  FPenWidth:=AValue;
  Changed;
end;

procedure TShapeObject.SetRHR(const AValue: Double);
begin
  if FRHR=AValue then exit;
  FRHR:=AValue;
  Changed;
end;

procedure TShapeObject.SetRVR(const AValue: Double);
begin
  if FRVR=AValue then exit;
  FRVR:=AValue;
  Changed;
end;

procedure TShapeObject.SetShapeType(const AValue: TShapeType);
begin
  if FShapeType=AValue then exit;
  FShapeType:=AValue;
  Changed;
end;

procedure TShapeObject.SetTransParent(const AValue: Boolean);
begin
  if FTransParent=AValue then exit;
  FTransParent:=AValue;
  Changed;
end;

class function TShapeObject.GetMementoClass: TMementoClass;
begin
  Result:=TShapeMemento;
end;

procedure TShapeObject.DoApplyMemento(AMemento: TMemento);

Var
  M : TShapeMemento;

begin
  inherited DoApplyMemento(AMemento);
  M:=TShapeMemento(AMemento);
  FPenColor:=M.FPenColor;
  FFillColor:=M.FFillColor;
  FPenWidth:=M.FPenWidth;
  FRHR:=M.FRHR;
  FRVR:=M.FRVR;
  FShapeType:=M.FShapeType;
  FTransParent:=M.FTransParent;
end;

procedure TShapeObject.DoFillMemento(AMemento: TMemento);

Var
  M : TShapeMemento;
begin
  inherited DoFillMemento(AMemento);
  M:=TShapeMemento(AMemento);
  M.FPenColor    := FPenColor;
  M.FFillColor   := FFillColor;
  M.FPenWidth    := FPenWidth;
  M.FRHR         := FRHR;
  M.FRVR         := FRVR;
  M.FShapeType   := FShapeType;
  M.FTransParent := FTransParent;
end;

procedure TShapeObject.DrawObject(ACanvas: TCanvas; PageRect: TRect;
  ADPI: Integer);

Var
  R : TRect;

begin
  R:=GetObjectRect(ACanvas,PageRect,ADPI);
  DrawShape(ACanvas,R,ADPI);
end;

procedure TShapeObject.DrawShape(ACanvas: TCanvas; R: TRect; ADPI : Integer);

Var
  DX,DY : Integer;

begin
  DX:=(R.Right-R.Left+1) div 2;
  DY:=(R.Bottom-R.Top+1) div 2;
  If not Transparent then
    begin
    ACanvas.Brush.Color:=FFillColor;
    ACanvas.Brush.Style:=bsSolid;
    ACanvas.FillRect(R);
    end;
  ACanvas.Pen.Color:=FPenColor;
  ACanvas.Pen.Width:=FPenWidth;
  Case ShapeType of
    stHorizontal : ACanvas.Line(R.Left,R.Top+DY,R.Right,R.Top+DY);
    stVertical   : ACanvas.Line(R.Left+DX,R.Top,R.Left+DX,R.Bottom);
    stDiagonal1  : ACanvas.Line(R.Left,R.Bottom,R.Right,R.Top);
    stDiagonal2  : ACanvas.Line(R.Left,R.Top,R.Right,R.Bottom);
    stEllipse    : begin
                   ACanvas.Ellipse(R);
                   If not TransParent then
                     ACanvas.FloodFill(R.Top+DY,R.Left+DY,FPenColor,fsSurface);
                   end;
    stRectangle     : begin
                   ACanvas.FillRect(R);
                   ACanvas.Rectangle(R);
                   end;
    stRoundedRectangle : begin
                    ACanvas.RoundRect(R,ToPoints(FRHR,ADPI),ToPoints(FRVR,ADPI));
                    If not TransParent then
                      ACanvas.FloodFill(R.Top+DY,R.Left+DY,FPenColor,fsSurface);
                   end;
  end;
end;

constructor TShapeObject.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FTransparent:=False;
  FFillColor:=clWhite;
  FPenColor:=clBlack;
  FPenWidth:=1;
  FShapeType:=stRectangle;
  FRHR:=0.5;
  FRVR:=0.5;
end;

{ TLabelObject }

procedure TLabelObject.SetCaption(const AValue: String);
begin
  if FCaption=AValue then exit;
  FCaption:=AValue;
  Changed;
end;

procedure TLabelObject.SetAlignment(const AValue: TAlignMent);
begin
  if FAlignMent=AValue then exit;
  FAlignMent:=AValue;
  Changed;
end;

procedure TLabelObject.SetFillColor(const AValue: TColor);
begin
  if FFillColor=AValue then exit;
  FFillColor:=AValue;
  Changed;
end;

procedure TLabelObject.SetFont(const AValue: TFont);
begin
  if FFont=AValue then exit;
  FFont.Assign(AValue);
  Changed;
end;

procedure TLabelObject.Setlayout(const AValue: TTextLayout);
begin
  if FLayout=AValue then exit;
  FLayout:=AValue;
  Changed;
end;

procedure TLabelObject.SetTransParent(const AValue: Boolean);
begin
  if FTransParent=AValue then exit;
  FTransParent:=AValue;
  Changed;
end;

procedure TLabelObject.SetWordWrap(const AValue: Boolean);
begin
  if FWordWrap=AValue then exit;
  FWordWrap:=AValue;
  Changed;
end;

class function TLabelObject.GetMementoClass: TMementoClass;
begin
  Result:=TLabelMemento;
end;

procedure TLabelObject.DoApplyMemento(AMemento: TMemento);

Var
  M : TLabelMemento;

begin
  inherited DoApplyMemento(AMemento);
  M:=TLabelMemento(AMemento);
  FAlignMent:=M.FAlignMent;
  FCaption:=M.FCaption;
  FFillColor:=M.FFillColor;
  FFont.Assign(M.FFont);
  FLayout:=M.FLayout;
  FTransParent:=M.FTransParent;
  FWordWrap:=M.FWordWrap;
end;

procedure TLabelObject.DoFillMemento(AMemento: TMemento);

Var
  M : TLabelMemento;

begin
  inherited DoFillMemento(AMemento);
  M:=TLabelMemento(AMemento);
  M.FFont.Assign(FFont);
  M.FAlignMent   := FAlignMent;
  M.FCaption     := FCaption;
  M.FFillColor   := FFillColor;
  M.FLayout      := FLayout;
  M.FTransParent := FTransParent;
  M.FWordWrap    := FWordWrap;
end;

procedure TLabelObject.DrawObject(ACanvas: TCanvas; PageRect: TRect;
  ADPI: Integer);

Var
  TT : TTextStyle;
  R : TRect;
begin
  ACanvas.Font:=Self.Font;
  R.Left:=PageRect.Left+ToPoints(Left,ADPI);
  R.Top:=PageRect.Top+ToPoints(Top,ADPI);
  R.Right:=R.Left+ToPoints(Width,ADPI);
  R.Bottom:=R.Top+ToPoints(Height,ADPI);
  FillChar(TT,SizeOf(TTextStyle),0);
  TT.AlignMent:=Self.AlignMent;
  TT.Layout:=Layout;
  TT.WordBreak:=WordWrap;
  TT.Opaque:=Not Transparent;
  ACanvas.TextRect(R,R.Left,R.Top,Caption,TT);
end;

constructor TLabelObject.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFont:=CreateDefaultFont;
  FCaption:='New label';
end;

destructor TLabelObject.Destroy;
begin
  FreeAndNil(FFont);
  inherited Destroy;
end;

{ TImageObject }

procedure TImageObject.SetPicture(const AValue: TPicture);
begin
  if FPicture=AValue then exit;
  FPicture.Assign(AValue);
end;

procedure TImageObject.SetCentered(const AValue: Boolean);
begin
  if FCentered=AValue then exit;
  FCentered:=AValue;
  Changed;
end;

procedure TImageObject.SetStretched(const AValue: Boolean);
begin
  if FStretched=AValue then exit;
  FStretched:=AValue;
  Changed;
end;

class function TImageObject.GetMementoClass: TMementoClass;
begin
  Result:=TImageMemento;
end;

procedure TImageObject.DoApplyMemento(AMemento: TMemento);

Var
  M : TImageMemento;

begin
  inherited DoApplyMemento(AMemento);
  M:=TImageMemento(AMemento);
  FPicture.Assign(M.FPicture);
  FCentered:=M.FCentered;
  FStretched:=M.FStretched;
end;

procedure TImageObject.DoFillMemento(AMemento: TMemento);

Var
  M : TImageMemento;

begin
  inherited DoFillMemento(AMemento);
  M:=TImageMemento(AMemento);
  M.FPicture.Assign(FPicture);
  M.FCentered  := FCentered;
  M.FStretched := FStretched;
end;

procedure TImageObject.DrawObject(ACanvas: TCanvas; PageRect: TRect;
  ADPI: Integer);

Var
  R : TRect;

begin
  R:=GetObjectRect(ACanvas,PageRect,ADPI);
  If Stretched then
    ACanvas.StretchDraw(R,FPicture.Graphic)
  else
    begin
    If Centered then
      begin
      If FPicture.Width<(R.Right-R.Left) then
        InflateRect(R,-((R.Right-R.Left-FPicture.Width) div 2),0);
      If FPicture.Height<(R.Bottom-R.Top) then
        InflateRect(R,0,-((R.Bottom-R.Top-FPicture.Height) div 2));
      end;
    ACanvas.ClipRect:=R;
    ACanvas.Clipping:=True;

    Try
      ACanvas.Draw(R.Left,R.Top,FPicture.Graphic);
    Finally
      ACanvas.Clipping:=False;
    end;
    end;
end;

constructor TImageObject.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPicture:=TPicture.Create;
end;

destructor TImageObject.Destroy;
begin
  FreeAndNil(FPicture);
  inherited Destroy;
end;

{ TInlayPage }

procedure TInlayPage.SetDepth(const AValue: Double);
begin
  if FDepth=AValue then exit;
  FDepth:=AValue;
  Changed;
end;

function TInlayPage.GetPageType: TPageType;
begin
  Result:=ptInlay;
end;

procedure TInlayPage.DrawInlay(ACanvas: TCanvas; PageRect: TRect; ADPI: Integer);

Var
  X : Integer;
  D : Integer;

begin
  Inherited;
  ACanvas.Pen.Width:=1;
  D:=ToPoints(FDepth,ADPI);
  X:=PageRect.Left+D;
  ACanvas.Line(X,PageRect.TOP,X,PageRect.Bottom);
  X:=PageRect.Right-D;
  ACanvas.Line(X,PageRect.TOP,X,PageRect.Bottom);
end;

constructor TInlayPage.create(AOwner: TComponent);
begin
  inherited create(AOwner);
  FDepth:=0.75;
end;

{ TDiscLabelPage }

procedure TDiscLabelPage.SetRadius1(const AValue: Double);
begin
  if FRadius1=AValue then exit;
  FRadius1:=AValue;
  Changed;
end;

procedure TDiscLabelPage.SetRadius2(const AValue: Double);
begin
  if FRadius2=GetRadius then exit;
  FRadius2:=AValue;
  Changed;
end;

function TDiscLabelPage.GetPageType: TPageType;
begin
  Result:=ptDiscLabel;
end;

function TDiscLabelPage.GetRadius: Double;
begin
  Result:=FRadius2;
  If (Result=0) then
    begin
    Result:=Width;
    If Height<Width then
      Result:=Height;
    Result:=Result/2;
    end;
end;

function TDiscLabelPage.GetRadiusStored: boolean;
begin
  Result:=(FRadius2>0);
end;

procedure TDiscLabelPage.DrawDiscLabel(ACanvas: TCanvas; PageRect: TRect;
  ADPI: Integer);

Var
  R : Integer;
  X0,Y0 : Integer;

begin
  inherited DrawDiscLabel(ACanvas, PageRect, ADPI);
  ACanvas.Pen.Color:=clBlack;
  ACanvas.Pen.Width:=1;
  ACanvas.Ellipse(PageRect);
  R:=ToPoints(Radius1,ADPI);
  X0:=(PageRect.Left+PageRect.Right) div 2;
  Y0:=(PageRect.Top+PageRect.Bottom) div 2;
  ACanvas.EllipseC(X0,Y0,R,R);
end;

constructor TDiscLabelPage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FRadius1:=2;
end;

{ TBookletBackPage }

function TBookletBackPage.GetPageType: TPageType;
begin
  Result:=ptBookletBack;
end;

{ TBookletFrontPage }

function TBookletFrontPage.GetPageType: TPageType;
begin
  Result:=ptBookletFront;
end;

{ Used in sorting horizontally. }
Function HCompare (P1,P2 : Pointer) : Integer;
Var
  O : TCoverObject;
  D : Double;
begin
  O:=TCoverObject(P1);
  D:=O.Left+O.Width/2;
  O:=TCoverObject(P2);
  D:=D-(O.Left+O.Width/2);
  If D>0 then
    Result:=1
  else if D<0 then
    result:=-1
  else
    Result:=0;
end;

Function VCompare (P1,P2 : Pointer) : Integer;
Var
  O : TCoverObject;
  D : Double;
begin
  O:=TCoverObject(P1);
  D:=O.Top+O.Height/2;
  O:=TCoverObject(P2);
  D:=D-(O.Top+O.Height/2);
  If D>0 then
    Result:=1
  else if D<0 then
    result:=-1
  else
    Result:=0;
end;

procedure TSelectionList.DeleteSelection;

Var
  O : TCoverobject;
  I : Integer;

begin
  For I:=Count-1 downto 0 do
    begin
    O:=CoverObjects[I];
    O.Free;
    Delete(i);
    end;
end;

procedure TSelectionList.AdjustSizes(Height: TSizeAdjust; HSize: Double;
  Width: TSizeAdjust; WSize: Double);

Var
  i : Longint;
  C : TCoverObject;
begin
  If Count<1 then
    exit;
  // Determine height and width
  C:=CoverObjects[0];
  If (Height in [saSmallest,saLargest]) then
    HSize:=C.Height;
  If (Width in [saSmallest,SaLargest]) Then
    WSize:=C.Width;
  For I:=1 to Count-1 do
    begin
    C:=CoverObjects[i];
    Case Height of
      saSmallest : If HSize>C.Height then HSize:=C.Height;
      saLargest : If HSize<C.Height then HSize:=C.Height;
    end;
    Case Width of
      saSmallest : If WSize>C.Width then WSize:=C.Width;
      saLargest : If WSize<C.Width then WSize:=C.Width
    end;
    end;
  // Set sizes
  For I:=0 to Count-1 do
    begin
    C:=CoverObjects[i];
    If (Height<>saNone) then C.Height:=HSize;
    If (Width<>saNone) then C.Width:=WSize;
    end;
end;

function TSelectionList.GetC(Index : Integer): TCoverObject;
begin
  Result:=TCoverObject(Items[Index]);
end;

function TSelectionList.GetHCenter: Double;
begin
  Result:=(HMin+HMax)/2
end;

function TSelectionList.GetVCenter: Double;
begin
  Result:=(VMin+VMax)/2
end;

procedure TSelectionList.SetPage(const AValue: TCoverPage);
begin
  FPage:=AValue;
  FPVCenter:=FPage.Height / 2;
  FPHCenter:=FPage.Width / 2;
end;


procedure TSelectionList.CalcAlignRect;

Var
  I : Integer;
  C : TCoverObject;

begin
  // Calculate align rectangle
  FHMin:=Page.Width;
  FVMin:=Page.Height;
  FHMax:=0;
  FVMax:=0;
  For I:=0 to Count-1 do
    begin
    C:=CoverObjects[i];
    If FHMin>C.Left then
      FHMin:=C.Left;
    If FVMin>C.Top then
      FVMin:=C.Top;
    If FHMax<C.Left+C.Width then
      FHMax:=C.Left+C.Width;
    If FVMax<C.Top+C.Height then
      FVMax:=C.Top+C.Height;
    end;

end;

Procedure TSelectionList.AlignControl (Index : Integer; Hor : THAlignAction; Ver : TValignAction);

Var
  C : TCoverObject;

begin
  C:=CoverObjects[Index];
  Case hor of
    haleft   : C.Left:=HMin;
    haRight  : C.Left:=HMax-C.Width;
    haCenter : C.Left:=HCenter-C.Width / 2;
    haCentP  : C.Left:=PageHCenter-C.Width / 2;
  end;
  Case Ver of
    vaTop    : C.Top:=VMin;
    vaBottom : C.Top:=VMax-C.Height;
    vaCenter : C.Top:=VCenter-C.Height/2;
    vaCentP  : C.Left:=PageVCenter-C.Height/2;
  end;
end;

Procedure TSelectionList.AlignObjects (Hor : THAlignAction; Ver : TValignAction);


Var
  SCenter,Delta : Double;
  I : longint;
  C : TCoverObject;

begin
  CalcAlignRect;
  If (Hor<>haSpace) and (Ver<>vaSpace) then
    For I:=0 to Count-1 do
      AlignControl(I,Hor,Ver)
  else
    begin
    // We must do two runs, because the sort is different for
    // Horizontal and vertical spacing.
    // Horizontal run
    If (Hor=haSpace) then
      begin
      Sort(@HCompare);
      C:=CoverObjects[0];
      SCenter:=C.Left+C.Width/2;
      C:=CoverObjects[Count-1];
      If Count>1 then
        Delta:=(C.Left+(C.Width/2) - SCenter)/Count-1
      else
        Delta:=0;
      // Set positions
      For I:=0 To Count-1 do
        begin
        C:=CoverObjects[i];
        If Not (Ver in [vaNone,vaSpace]) then
          AlignControl(I,haNone,Ver);
        C.Left:=SCenter-C.Width/2;
        SCenter:=SCenter+Delta;
        end;
      end;
    // Vertical run.
    If (Ver=VaSpace) then
      begin
      Sort(@VCompare);
      C:=CoverObjects[0];
      SCenter:=C.Top+C.Height/2;
      C:=CoverObjects[Count-1];
      If Count>1 then
        Delta:=(C.Top+(C.Width/2) - SCenter)/Count-1
      else
        Delta:=0;
      // Set positions
      For I:=0 To Count-1 do
        begin
        C:=CoverObjects[i];
        If Not (Hor in [haNone,haSpace]) then
          AlignControl(I,Hor,vaNone);
        C.Top:=SCenter-C.Height/2;
        SCenter:=SCenter+Delta;
        end;
      end;
    end;
end;


{ TMemento }

constructor TMemento.CreateMemento;
begin
  // Do nothing
end;

{ TLabelMemento }

constructor TLabelMemento.CreateMemento;
begin
  inherited CreateMemento;
  FFont:=TFont.Create;
end;

destructor TLabelMemento.Destroy;

begin
  FreeAndNil(FFont);
  inherited Destroy;
end;

{ TImageMemento }

constructor TImageMemento.CreateMemento;
begin
  inherited CreateMemento;
  FPicture:=TPicture.Create;
end;

destructor TImageMemento.Destroy;
begin
  FreeAndNil(FPicture);
  inherited Destroy;
end;

{ TTrackListMemento }

constructor TTrackListMemento.CreateMemento;

Var
  I : Integer;

begin
  inherited CreateMemento;
  FFont:=TFont.Create;
  For I:=0 to 3 do
    FTrackElements[i]:=TTrackElement.Create;
end;

destructor TTrackListMemento.Destroy;

Var
  I : Integer;

begin
  FreeAndNil(FFont);
  For I:=0 to 3 do
    FreeAndNil(FTrackElements[i]);
  inherited Destroy;
end;

{ TCoverComponent }

class function TCoverComponent.GetMementoClass: TMementoClass;
begin
  Result:=TMemento;
end;

procedure TCoverComponent.DoApplyMemento(AMemento: TMemento);
begin
  // Do nothing
end;

procedure TCoverComponent.DoFillMemento(AMemento: TMemento);
begin
  // Do nothing
end;

function TCoverComponent.GetMemento: TMemento;
begin
  Result:=GetMementoClass.CreateMemento;
  DoFillMemento(Result)
end;

procedure TCoverComponent.ApplyMemento(AMemento: TMemento);
begin
  If Not (AMemento is GetMementoClass) then
    Raise Exception.CreateFmt(SErrMementoNotApplicable,[AMemento.ClassName,ClassName]);
  DoApplyMemento(AMemento);
  Changed;
end;

{ TSelection }

procedure TSelection.GetChildren(Proc: TGetChildProc; Root: TComponent);

Var
  I : Integer;

begin
  If Assigned(Selection) then
    For I:=0 to Selection.Count-1 do
      Proc(FSelection.CoverObjects[i]);
end;

{ TCDCoverAction }

function TCDCoverAction.Description: String;
begin
  Result:=DisplayName;
end;

{ TCDCoverActions }

function TCDCoverActions.GetA(Index : Integer): TCDCoverAction;
begin
  Result:=Factions[Index] as TCDCoverAction;
end;

function TCDCoverActions.GetActionCount: Integer;
begin
  Result:=Factions.Count;
end;

procedure TCDCoverActions.SetMaxCount(const AValue: Integer);
begin
  if FMaxCount=AValue then exit;
  FMaxCount:=AValue;
  While ActionCount>FMaxCount do
    FActions.Delete(0);
end;

constructor TCDCoverActions.Create(AMaxCount: Integer);
begin
  FActions:=TObjectList.Create(True);
  FMaxCount:=AMaxCount;
end;

destructor TCDCoverActions.Destroy;
begin
  FreeAndNil(FActions);
  inherited Destroy;
end;

function TCDCoverActions.UndoLastAction: Boolean;

Var
  A : TCDCoverAction;
  I : Integer;
begin
  A:=LastAction;
  Result:=A<>Nil;
  If not Result then
    Exit;
  A.Undo;
  // This will free A.
  FActions.Remove(A);
end;

function TCDCoverActions.LastAction: TCDCoverAction;

Var
  I : Integer;

begin
  I:=ActionCount-1;
  If I<0 then
    Result:=Nil
  else
    Result:=GetA(I);
end;

function TCDCoverActions.AddAction(AAction: TCDCoverAction): Integer;
begin
  If ActionCount=MaxCount Then
    FActions.Delete(0);
  FActions.Add(AAction);
end;


{ TSingleObjectAction }

constructor TSingleObjectAction.Create(AObject: TCoverObject);
begin
  FObject:=AObject;
end;

{ TSelectedObjectsAction }

constructor TSelectedObjectsAction.Create(ASelection: TSelectionlist);

Var
  I : Integer;

begin
  FObjects:=TSelectionList.Create;
  For I:=0 to ASelection.Count-1 do
    FObjects.Add(ASelection.CoverObjects[i]);
end;

destructor TSelectedObjectsAction.Destroy;
begin
  FreeAndNil(FObjects);
  inherited Destroy;
end;


{ TAddObjectAction }


procedure TAddObjectAction.Undo;
begin
  FreeAndNil(FObject);
end;

function TAddObjectAction.DisplayName: String;
begin
  Result:='Add object';
end;

{ TEditObjectAction }

constructor TEditObjectAction.Create(AObject: TCoverObject);
begin
  inherited Create(AObject);
  FMemento:=AObject.GetMemento;
end;

destructor TEditObjectAction.Destroy;
begin
  FreeAndNil(FMemento);
  inherited Destroy;
end;

procedure TEditObjectAction.Undo;
begin
  CoverObject.ApplyMemento(FMemento);
end;

function TEditObjectAction.DisplayName: String;
begin
  Result:='Edit object';
end;

{ TResizeObjectAction }

constructor TResizeObjectAction.Create(AObject: TCoverObject);
begin
  inherited Create(AObject);
  FDims:=AObject.FDims;
end;

procedure TResizeObjectAction.Undo;
begin
  CoverObject.FDims:=FDims;
end;

function TResizeObjectAction.DisplayName: String;
begin
  Result:='Resize object';
end;

{ TDeleteObjectsAction }

constructor TDeleteObjectsAction.Create(ASelection: TSelectionList);


begin
  inherited Create(ASelection);
  If (ASelection.Count>0) then
    FParent:=ASelection[0].GetParentComponent as TCoverPage;
end;

Destructor TDeleteObjectsAction.Destroy;

Var
  I : Integer;

begin
  For I:=0 to Objects.Count-1 do
    Objects[i].Free;
  inherited Destroy;
end;

procedure TDeleteObjectsAction.Undo;

Var
  I : Integer;
  O : TCoverObject;
  C : TCDCOver;

begin
  If Assigned(FParent) then
    begin
    C:=FParent.Owner as TCDCover;
    For I:=Objects.Count-1 downto 0 do
      begin
      O:=Objects.CoverObjects[I];
      C.InsertComponent(O);
      O.SetParentComponent(FParent);
      Objects.Delete(i);
      end;
    end;
end;

function TDeleteObjectsAction.DisplayName: String;
begin
  Result:='Delete objects';
end;

{ TMoveObjectsAction }

constructor TMoveObjectsAction.Create(ASelection: TSelectionList);

Var
  I : Integer;

begin
  inherited Create(ASelection);
  FOrigins:=GetMem(ASelection.Count*SizeOf(TObjectOrigin));
  For I:=0 to ASelection.Count-1 do
    begin
    FOrigins[I].X:=ASelection[i].Left;
    FOrigins[I].Y:=ASelection[i].Top;
    end;
end;

destructor TMoveObjectsAction.Destroy;
begin
  FreeMem(FOrigins);
  Inherited;
end;

procedure TMoveObjectsAction.Undo;

Var
  I : Integer;

begin
  For I:=0 to Objects.Count-1 do
    begin
    Objects[i].Left:=FOrigins[I].X;
    Objects[i].Top:=FOrigins[I].Y;
    end;
end;

function TMoveObjectsAction.DisplayName: String;
begin
  Result:='Move objects';
end;

{ TAlignObjectsAction }

function TAlignObjectsAction.DisplayName: String;
begin
  Result:='Align objects';
end;

{ TResizeObjectsAction }

constructor TResizeObjectsAction.Create(ASelection: TSelectionList);
Var
  I : Integer;

begin
  inherited Create(ASelection);
  FDimensions:=GetMem(ASelection.Count*SizeOf(TObjectDimensions));
  For I:=0 to ASelection.Count-1 do
    FDimensions[I]:=ASelection[i].FDims;
end;

destructor TResizeObjectsAction.Destroy;
begin
  FreeMem(FDimensions);
  inherited Destroy;
end;

procedure TResizeObjectsAction.Undo;

Var
  I : Integer;

begin
  For I:=0 to Objects.Count-1 do
    Objects[i].FDims:=FDimensions[i];
end;

function TResizeObjectsAction.DisplayName: String;
begin
  Result:='Resize objects';
end;

initialization
  RegisterClasses([TCoverPage,TBookletFrontPage,TBookletBackPage, TInlayPage,TDiscLabelPage, TCDDisc,TCDCover,TLabelObject,TShapeObject,TTrackList, TSelection]);
end.

