unit coverconfig;

{$mode objfpc}{$H+}
{ $define usedelegation}

interface

uses
  Classes, SysUtils, fpobserver, inifiles;

Type

  { TCDConverConfiguration }

  { TCDCoverConfiguration }

  TCDCoverConfiguration = Class(TObservedComponent)
  private
    FCacheDir: String;
    FCDDevice: String;
    FHostURL: String;
    FUseCache: Boolean;
    FUser: String;
    procedure SetCacheDir(const AValue: String);
    procedure SetCDDevice(const AValue: String);
    procedure SetHostURL(const AValue: String);
    procedure SetUseCache(const AValue: Boolean);
    procedure SetUser(const AValue: String);
  Protected
    Procedure Changed;
  Public
    Procedure LoadConfig;
    Procedure SaveConfig;
  Published
    Property CDDevice : String Read FCDDevice Write SetCDDevice;
    Property CDDBUser : String Read FUser Write SetUser;
    Property CDDBHostURL : String Read FHostURL Write SetHostURL;
    Property CDDBUseCache : Boolean Read FUseCache Write SetUseCache;
    Property CDDBCacheDir : String Read FCacheDir Write SetCacheDir;
  end;

  ECDCoverConfig = Class(Exception);

  { TRecentFile }

  TRecentFile = Class(TObservedCollectionItem)
  private
    FFileName: String;
    FPerformer: String;
    FTitle: String;
    procedure SetFileName(const AValue: String);
    procedure SetPerformer(const AValue: String);
    procedure SetTitle(const AValue: String);
  Protected
    Procedure LoadFromIni(Ini : TCustomIniFile; Const ASection : String);
    Procedure SaveToIni(Ini : TCustomIniFile; Const ASection : String);
  Published
    Property FileName : String Read FFileName Write SetFileName;
    Property Title : String Read FTitle Write SetTitle;
    Property Performer : String Read FPerformer Write SetPerformer;
  end;

  { TRecentFiles }

  TRecentFiles = Class(TObservedCollection)
  private
    function GetR(Aindex : Integer): TRecentFile;
    procedure SetR(Aindex : Integer; const AValue: TRecentFile);
  Public
    Procedure LoadFromIni(Ini : TCustomIniFile);
    Procedure SaveToIni(Ini : TCustomIniFile);
    Procedure LoadFromFile(AFileName : String);
    Procedure SaveToFile(AFileName : String);
    Function IndexOfFileName(Const AFileName : String) : Integer;
    Property RecentFile[Aindex : Integer]: TRecentFile Read GetR Write SetR;default;
  end;

implementation

uses cddbquery;

Resourcestring
  SNoConfigDir       = 'Could not create the configuration directory.';

Const
  SConfig     = 'Config';
  KeyDevice   = 'Device';
  KeyUser     = 'User';
  KeyHost     = 'Host';
  KeyCacheDir = 'CacheDir';
  KeyUseCache = 'UseCache';


{ TCDConverConfiguration }

procedure TCDCoverConfiguration.SetCacheDir(const AValue: String);
begin
  if FCacheDir=AValue then exit;
  FCacheDir:=AValue;
  Changed;
end;


procedure TCDCoverConfiguration.SetCDDevice(const AValue: String);
begin
  if FCDDevice=AValue then exit;
  FCDDevice:=AValue;
  Changed;
end;

procedure TCDCoverConfiguration.SetHostURL(const AValue: String);
begin
  if FHostURL=AValue then exit;
  FHostURL:=AValue;
  Changed;
end;

procedure TCDCoverConfiguration.SetUseCache(const AValue: Boolean);
begin
  if FUseCache=AValue then exit;
  FUseCache:=AValue;
  Changed;
end;

procedure TCDCoverConfiguration.SetUser(const AValue: String);
begin
  if FUser=AValue then exit;
  FUser:=AValue;
  Changed;
end;

procedure TCDCoverConfiguration.Changed;
begin
{$ifdef usedelegation}
  ObservedHook.NotifyObservers(Self,ooChanged);
{$else}
  NotifyObservers(Self,ooChanged);
{$endif}
end;

procedure TCDCoverConfiguration.LoadConfig;

Var
  D,FN : String;
  Ini : TIniFile;

begin
  D:=IncludeTrailingPathDelimiter(GetAppConfigDir(False)+'cddb');
  FN:=GetAppConfigFile(False);
  If not ForceDirectories(ExtractFilePath(FN)) then
    Raise ECDCoverConfig.Create(SNoConfigDir);
  Ini:=TIniFile.Create(FN);
  try
    FCDDevice:=Ini.ReadString(SConfig,KeyDevice,'');
    FUser:=Ini.ReadString(SConfig,KeyUser,'');
    FHostURL:=Ini.ReadString(SConfig,KeyHost,DefaultHost);
    FuseCache:=Ini.ReadBool(SConfig,KeyUseCache,False);
    FCacheDir:=Ini.ReadString(SConfig,KeyCacheDir,D);
  finally
    Ini.Free;
  end;
  Changed;
end;

procedure TCDCoverConfiguration.SaveConfig;

Var
  FN : String;
  Ini : TIniFile;

begin
  FN:=GetAppConfigFile(False);
  If not ForceDirectories(ExtractFilePath(FN)) then
    Raise ECDCoverConfig.Create(SNoConfigDir);
  Ini:=TIniFile.Create(FN);
  With Ini Do
  try
    WriteString(SConfig,KeyDevice,FCDDevice);
    WriteString(SConfig,KeyUser,FUser);
    WriteString(SConfig,KeyHost,FHostURL);
    WriteBool(SConfig,KeyUseCache,FuseCache);
    WriteString(SConfig,KeyCacheDir,FCacheDir);
  finally
    Ini.Free;
  end;
end;

{ TRecentFile }

procedure TRecentFile.SetFileName(const AValue: String);
begin
  if FFileName=AValue then exit;
  FFileName:=AValue;
  Change;
end;

procedure TRecentFile.SetPerformer(const AValue: String);
begin
  if FPerformer=AValue then exit;
  FPerformer:=AValue;
  Change;
end;

procedure TRecentFile.SetTitle(const AValue: String);
begin
  if FTitle=AValue then exit;
  FTitle:=AValue;
  Change;
end;

procedure TRecentFile.LoadFromIni(Ini: TCustomIniFile; const ASection: String);
begin
  With Ini do
    begin
    FFileName:=ReadString(ASection,'FileName','');
    FPerformer:=ReadString(ASection,'Performer','');
    FTitle:=ReadString(ASection,'Title','');
    end;
end;

procedure TRecentFile.SaveToIni(Ini: TCustomIniFile; const ASection: String);
begin
  With Ini do
    begin
    WriteString(ASection,'FileName',FFileName);
    WriteString(ASection,'Performer',FPerformer);
    WriteString(ASection,'Title',FTitle);
    end;
end;

{ TRecentFiles }

function TRecentFiles.GetR(Aindex : Integer): TRecentFile;
begin
  Result:=TRecentFile(Items[Aindex])
end;

procedure TRecentFiles.SetR(Aindex : Integer; const AValue: TRecentFile);
begin
  Items[Aindex]:=AValue;
end;

procedure TRecentFiles.LoadFromIni(Ini: TCustomIniFile);

Var
  I,ACount : integer;
  R : TRecentFile;

begin
  Clear;
  ACount:=Ini.ReadInteger('Recent','Count',0);
  For I:=1 to ACount do
    begin
    R:=Add as TRecentFile;
    R.LoadFromIni(Ini,'Recent_'+IntToStr(I));
    end;
end;

procedure TRecentFiles.SaveToIni(Ini: TCustomIniFile);

Var
  I,ACount : integer;
  R : TRecentFile;

begin
  ACount:=Ini.ReadInteger('Recent','Count',0);
  If ACount>Count then
    For I:=Count+1 to ACount do
      Ini.EraseSection('Recent_'+IntToStr(I));
  Ini.WriteInteger('Recent','Count',Count);
  For I:=1 to Count do
    begin
    R:=RecentFile[i-1];
    R.SaveToIni(Ini,'Recent_'+IntToStr(I));
    end;
end;

procedure TRecentFiles.LoadFromFile(AFileName: String);

Var
  Ini : TMemIniFile;

begin
  Ini:=TMemIniFile.Create(AFileName);
  try
    LoadFromIni(Ini);
  finally
    Ini.Free;
  end;
end;

procedure TRecentFiles.SaveToFile(AFileName: String);

Var
  Ini : TMemIniFile;

begin
  Ini:=TMemIniFile.Create(AFileName);
  try
    SaveToIni(Ini);
  finally
    Ini.Free;
  end;
end;

function TRecentFiles.IndexOfFileName(const AFileName: String): Integer;

Var
  FN : String;

begin
  FN:=ExpandFileName(AFileName);
  Result:=Count-1;
  While (Result>=0) and (Not SameFileName(FN,GetR(Result).FileName)) do
    Dec(Result);
end;

end.

