unit covermediators;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, lclmediators, fpobserver, typinfo, graphics, stdctrls,
  extctrls, cdcovers, forms, lclintf, graphtype;

Type  { TAlignmentMediator }

  TAlignmentMediator = Class(TSpeedButtonMediator)
  private
    FAlignMent: TAlignment;
    procedure SetAlignment(const AValue: TAlignment);
  protected
    Function GetBooleanValue : Boolean; override;
    Procedure SetBooleanValue(AValue : Boolean); override;
  Public
    Property Alignment : TAlignment Read FAlignMent Write SetAlignment;
  end;

  { TLayoutMediator }

  TLayoutMediator = Class(TSpeedButtonMediator)
  private
    FLayout: TTextLayout;
    procedure SetLayout(const AValue: TTextLayout);
  protected
    Function GetBooleanValue : Boolean; override;
    Procedure SetBooleanValue(AValue : Boolean); override;
  Public
    Property Layout : TTextLayout Read FLayout Write SetLayout;
  end;

  { TPanelShapeMediator }

  TPanelShapeMediator = Class(TControlMediator)
  private
    FPanel: TPanel;
    procedure DoPaint(Sender: TObject);
    procedure SetPanel(const AValue: TPanel);
  Protected
    Class Function PublishedViewProperty : Boolean; override;
    Class Function PublishedSubjectProperty : Boolean; override;
    function GetView: TObject; override;
    procedure SetupControl; override;
    Procedure DoObjectToView; override;
    Procedure DoViewToObject; override;
  Public
    Constructor Create(AOwner : TComponent); override;
    Property Control : TPanel Read FPanel Write SetPanel;
  end;

  { TFontMediator }

  TFontMediator = Class(TControlMediator)
  private
    FPanel: TPanel;
    procedure SetPanel(const AValue: TPanel);
  Protected
    Procedure DoObjectToView; override;
    Procedure DoViewToObject; override;
  Public
    Constructor Create(AOwner : TComponent); override;
    Property Control : TPanel Read FPanel Write SetPanel;

  end;

implementation

{ TAlignmentMediator }

procedure TAlignmentMediator.SetAlignment(const AValue: TAlignment);
begin
  if FAlignMent=AValue then exit;
  FAlignMent:=AValue;
  MaybeObjectToView;
end;

function TAlignmentMediator.GetBooleanValue: Boolean;
begin
  Result:=TAlignment(GetOrdProp(Subject,SubjectpropertyName))=Alignment;
end;

procedure TAlignmentMediator.SetBooleanValue(AValue: Boolean);
begin
  If AValue then
    SetOrdProp(Subject,SubjectpropertyName,Ord(Alignment));
end;

{ TLayoutMediator }

procedure TLayoutMediator.SetLayout(const AValue: TTextLayout);
begin
  If (AValue=FLayout) then exit;
  FLayout:=AValue;
  MaybeObjectToView;
end;

function TLayoutMediator.GetBooleanValue: Boolean;
begin
  Result:=TTextLayout(GetOrdProp(Subject,SubjectpropertyName))=Layout;
end;

procedure TLayoutMediator.SetBooleanValue(AValue: Boolean);
begin
  If AValue then
    SetOrdProp(Subject,SubjectpropertyName,Ord(Layout));
end;

{ TPanelShapeMediator }

procedure TPanelShapeMediator.SetPanel(const AValue: TPanel);
begin
  SetControl(FPanel,AValue);
end;

class function TPanelShapeMediator.PublishedViewProperty: Boolean;
begin
  Result:=False;
end;

class function TPanelShapeMediator.PublishedSubjectProperty: Boolean;
begin
  Result:=False;
end;

function TPanelShapeMediator.GetView: TObject;
begin
  Result:=FPanel;
end;

procedure TPanelShapeMediator.DoPaint(Sender : TObject);

Var
  DPI : Integer;
  R : TRect;
  B : Integer;

begin
  If Assigned(FPanel) and Assigned(Subject) then
    begin
    R:=FPanel.ClientRect;
    B := FPanel.BorderWidth;
    if (FPanel.BevelOuter <> bvNone) then
      Inc(B, FPanel.BevelWidth);
    if (FPanel.BevelInner <> bvNone) then
      Inc(B, FPanel.BevelWidth);
    inflaterect(R,-B,-B);
    FPanel.Canvas.Brush.Style:=bsSolid;
    FPanel.Canvas.Brush.Color:=FPanel.Color;
    FPanel.Canvas.FillRect(R);
    (Subject as TShapeObject).DrawShape(FPanel.Canvas,R,Screen.PixelsPerInch);
    end;
end;

procedure TPanelShapeMediator.SetupControl;
begin
  Inherited;
  FPanel.OnPaint:=@DoPaint;
end;

procedure TPanelShapeMediator.DoObjectToView;
begin
  FPanel.RePaint;
end;

procedure TPanelShapeMediator.DoViewToObject;
begin
  // Do nothing;
end;

constructor TPanelShapeMediator.Create(AOwner: TComponent);
begin
  Inherited;
  SubjectPropertyName:='ShapeType';
  ViewPropertyName:='Canvas';
  ReadOnly:=True;
end;

{ TFontMediator }

procedure TFontMediator.SetPanel(const AValue: TPanel);
begin
  SetControl(FPanel,AValue);
  FPanel:=AValue;
end;

procedure TFontMediator.DoObjectToView;

Var
  F : TObject;

begin
  F:=GetObjectprop(Subject,SubjectPropertyName,TFont);
  SetObjectprop(FPanel,ViewPropertyName,F);
end;

procedure TFontMediator.DoViewToObject;

Var
  F : TObject;

begin
  F:=GetObjectprop(FPanel,ViewPropertyName,TFont);
  SetObjectprop(Subject,SubjectPropertyName,F);
end;

constructor TFontMediator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SubjectPropertyName:='Font';
  ViewPropertyName:='Font';
end;

end.

