{ COPYRIGHT 2008 Michael Van Canneyt und Toolbox Verlag }
unit fratracklist;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, StdCtrls, ExtCtrls, Dialogs,
  fpobserver, cdcovers, lclmediators, CoverMediators, fracoverobject, CheckLst,
  Spin;

type
  TFrame = TCoverObjectFrame;
  { TTrackListFrame }

  TTrackListFrame = class(TFrame,IFPObserver)
    CBDisc: TComboBox;
    CLBShow: TCheckListBox;
    CBStandardFont: TCheckBox;
    EFormat: TEdit;
    FDTrackList: TFontDialog;
    Formatting: TLabel;
    Label2: TLabel;
    PSample: TPanel;
    LWidth: TLabel;
    LPosition: TLabel;
    SEWidth: TFloatSpinEdit;
    Label1: TLabel;
    LElements: TLabel;
    LPFont: TLabel;
    PListFont: TPanel;
    SEPosition: TSpinEdit;
    procedure CLBShowClick(Sender: TObject);
    procedure CLBShowClickCheck(Sender: TObject);
    procedure CLBShowItemClick(Sender: TObject; Index: integer);
    procedure PListFontDblClick(Sender: TObject);
  private
    { private declarations }
    FDiscMediator : TObjectComboBoxMediator;
    FFontmediator : TFontMediator;
    FWidthMediator : TFloatSpinEditMediator;
    FPositionMediator : TSpinEditMediator;
    FFormatMediator : TEditMediator;
    FStandardFontMediator : TCheckBoxMediator;
    FTrackFontMediator  : TFontMediator;
    FDiscs : TObservedObjectList;
    FCurrentElement : TTRackElement;
    FLastPosition : Integer;
    FBuilding : Boolean;
    procedure MaybeRebuildList;
    procedure SetCurrentElement(const AValue: TTrackElement);
    procedure SetupTrackMediators;
    procedure ShowElement(E: TTrackElement);
    procedure ShowElements;
  Protected
    function GetTabCaption: String; override;
    procedure SetCoverObject(const AValue: TCoverObject); override;
  public
    { public declarations }
    destructor Destroy; override;
    Procedure InitFrame; override;
    Procedure ObservedChanged(ASender : TObject; Operation : TObservedOperation);
    Property CurrentElement : TTrackElement Read FCurrentElement Write SetCurrentElement;
  end;

implementation

{ TTrackListFrame }

procedure TTrackListFrame.PListFontDblClick(Sender: TObject);
begin
  With FDTrackList do
    begin
    Font:=(CoverObject as TTrackList).Font;
    If Execute then
      (CoverObject as TTrackList).Font:=Font;
    end;
end;

procedure TTrackListFrame.CLBShowItemClick(Sender: TObject; Index: integer);
begin
end;

procedure TTrackListFrame.CLBShowClick(Sender: TObject);
begin
  if not FBuilding then
    With CLBShow do
      ShowElement((Items.Objects[ItemIndex]) as TTrackElement);
end;

procedure TTrackListFrame.CLBShowClickCheck(Sender: TObject);

Var
  E : TTrackElement;

begin
  if not FBuilding then
    With CLBShow do
      begin
      E:=(Items.Objects[ItemIndex]) as TTrackElement;
      E.Visible:=Checked[ItemIndex];
      ShowElement(E);
      end;
end;

function TTrackListFrame.GetTabCaption: String;
begin
  Result:='Track list';
end;

procedure TTrackListFrame.SetCoverObject(const AValue: TCoverObject);

begin
  FDiscs:=TObservedObjectList.CreateObserved(False);
  (AValue.Owner as TCDCover).GetDiscs(FDiscs);
  FDiscMediator.ValueList:=FDiscs;
  inherited SetCoverObject(AValue);
  ShowElements;
end;

destructor TTrackListFrame.Destroy;
begin
  FreeAndNil(FDiscs);
  CurrentElement:=Nil;
  inherited Destroy;
end;

procedure TTrackListFrame.ShowElements;


Var
  T : TTrackList;
  TE : TTRackElement;
  I,II : Integer;

begin
  FBuilding:=True;
  try
    T:=(CoverObject as TTrackList);
    II:=0;
    For I:=0 to 3 do
      begin
      TE:=T.Elements[i];
      CLBShow.Items[TE.Position-1]:=ElementTypeName(TE.ElementType);
      CLBShow.Items.Objects[TE.Position-1]:=TE;
      CLBShow.Checked[TE.Position-1]:=TE.Visible;
      IF TE=FCurrentElement then
        II:=TE.Position-1;
      end;
    CLBShow.ItemIndex:=II;
    If (FCurrentElement<>Nil) then
      ShowElement(FCurrentElement)
    else
      ShowElement(TTrackElement(CLBShow.Items.Objects[0]));
  Finally
    FBuilding:=False;
  end;
end;

procedure TTrackListFrame.ShowElement(E : TTrackElement);

begin
  If (FPositionMediator=Nil) then
    SetupTrackMediators;
  FWidthMediator.Subject:=E;
  FPositionMediator.Subject:=E;
  FFormatMediator.Subject:=E;
  FStandardFontMediator.Subject:=E;
  FTrackFontMediator.Subject:=E;
  CurrentElement:=E;
end;

procedure TTrackListFrame.SetupTrackMediators;

begin
  FWidthMediator:=TFloatSpinEditMediator.Create(Self);
  FWidthMediator.Control:=SEWidth;
  FWidthMediator.SubjectpropertyName:='Width';
  FWidthMediator.Active:=True;
  FPositionMediator:=TSpinEditMediator.Create(Self);
  FPositionMediator.Control:=SEPosition;
  FPositionMediator.SubjectpropertyName:='Position';
  FPositionMediator.Active:=True;
  FFormatMediator:=TEditMediator.Create(Self);
  FFormatMediator.Control:=EFormat;
  FFormatMediator.SubjectpropertyName:='Format';
  FFormatMediator.Active:=True;
  FStandardFontMediator:=TCheckBoxMediator.Create(Self);
  FStandardFontMediator.Control:=CBStandardFont;
  FStandardFontMediator.SubjectpropertyName:='ParentFont';
  FStandardFontMediator.Active:=True;
  FTrackFontMediator:=TFontMediator.Create(Self);
  FTrackFontmediator.Control:=PSample;
  FTrackFontmediator.Active:=True;
end;

procedure TTrackListFrame.SetCurrentElement(const AValue: TTrackElement);

Var
  I : IFPObserved;

begin
  If AVAlue=FCurrentElement then exit;
  If FCurrentElement<>Nil then
    If FCurrentElement.GetInterface(SGUIDObserved,I) then
      I.DetachObserver(Self);
  FCurrentElement:=AValue;
  If FCurrentElement<>Nil then
    begin
    If FCurrentElement.GetInterface(SGUIDObserved,I) then
      I.AttachObserver(Self);
    FLastPosition:=FCurrentElement.Position;
    end;
end;


procedure TTrackListFrame.InitFrame;
begin
  inherited InitFrame;
  FDiscMediator:=TObjectComboBoxMediator.Create(Self);
  FDiscMediator.SubjectPropertyName:='Disc';
  FDiscMediator.Control:=CBDisc;
  FFontmediator:=TFontMediator.Create(Self);
  FFontmediator.Control:=PListFont;
end;

procedure TTrackListFrame.ObservedChanged(ASender: TObject;
  Operation: TObservedOperation);
begin
  If ASender=FCurrentElement then
    If operation=oochanged then
      MaybeRebuildList
    else if operation=ooFree then
      FCurrentElement:=Nil;
end;

procedure TTrackListFrame.MaybeRebuildList;

begin
  If FCurrentElement.Position<>FLastPosition then
    ShowElements;
end;

initialization
  {$I fratracklist.lrs}

end.

