unit frmDiscEditor;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  ComCtrls, ExtCtrls, StdCtrls, ActnList, fpobserver, cdcovers, variants,
  typinfo, coverconfig, lclmediators, cddbquery, EditBtn;

type

  { TTreeNodeMediator }

  TTreeNodeMediator = Class(TBaseMediator)
  private
    FTreeNode: TTreeNode;
    procedure SetTreeNode(const AValue: TTreeNode);
  Protected
    Class Function PublishedViewProperty : Boolean; override;
    function GetView : TObject; override;
    Procedure DoViewToObject; override;
    Procedure DoObjectToView; override;
  Public
    Constructor Create(AOwner : TComponent); override;
    property TreeNode : TTreeNode Read FTreeNode Write SetTreeNode;
  end;


  { TTimeEditMediator }

  TTimeEditMediator = Class(TEditMediator)
  Protected
    Procedure DoViewToObject; override;
    Procedure DoObjectToView; override;
  end;


  { TDiscEditorForm }

  TDiscEditorForm = class(TForm)
    AAddDisc: TAction;
    AAddTrack: TAction;
    ATrackDown: TAction;
    ATrackUp: TAction;
    ADeleteTrack: TAction;
    AScanDisc: TAction;
    ADeleteDisc: TAction;
    ALDiscs: TActionList;
    BOK: TButton;
    EDuration: TEdit;
    EPerformer: TEdit;
    ETitle: TEdit;
    ILDiscs: TImageList;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    LETitle: TLabel;
    MComment: TMemo;
    Panel1: TPanel;
    PDetails: TPanel;
    Splitter1: TSplitter;
    TBDiscs: TToolBar;
    TBAddDisc: TToolButton;
    TBDeleteDisc: TToolButton;
    TBScan: TToolButton;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    TBAddTrack: TToolButton;
    TBDeleteTrack: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    TVDiscs: TTreeView;
    procedure AAddDiscExecute(Sender: TObject);
    procedure AAddTrackExecute(Sender: TObject);
    procedure AAddTrackUpdate(Sender: TObject);
    procedure ADeleteDiscExecute(Sender: TObject);
    procedure ADeleteDiscUpdate(Sender: TObject);
    procedure ADeleteTrackExecute(Sender: TObject);
    procedure ADeleteTrackUpdate(Sender: TObject);
    procedure AScanDiscExecute(Sender: TObject);
    procedure ATrackDownExecute(Sender: TObject);
    procedure ATrackDownUpdate(Sender: TObject);
    procedure ATrackUpExecute(Sender: TObject);
    procedure ATrackUpUpdate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure TVDiscsSelectionChanged(Sender: TObject);
  private
    FCDCover: TCDCover;
    FConfig: TCDCoverConfiguration;
    FCurrentDisc: TCDDisc;
    FCurrentTrack: TCDCoverTrack;
    FCurrentDiscNode: TTreeNode;
    FCurrentTrackNode: TTreeNode;
    FTitleMediator : TEditMediator;
    FPerformerMediator : TEditMediator;
    FDurationMediator : TEditMediator;
    FCommentMediator : TMemoMediator;
    function AddDiscToTree(ADisc: TCDDisc): TTreeNode;
    function AddTrackToTree(AParent: TTreeNode; ATrack: TCDCoverTrack
      ): TTreeNode;
    procedure DisplayCover;
    procedure DisplayDisc(Node: TTreeNode; ADisc: TCDDisc);
    procedure DisplayRoot;
    procedure DisplayTrack(Node: TTreeNode; ATrack: TCDCoverTrack);
    function FindNodeForDisc(D: TCDDisc): TTreeNode;
    procedure MoveTrack(Step: Integer);
    procedure ScanDisc;
    procedure SetCDCover(const AValue: TCDCover);
    { private declarations }
  public
    { public declarations }
    Function AddTrack : TCDCoverTrack;
    Function AddDisc: TCDDisc;
    Procedure DeleteDisc;
    Procedure DeleteTrack;
    Property CurrentDisc : TCDDisc Read FCurrentDisc;
    Property CurrentTrack : TCDCoverTrack Read FCurrentTrack;
    Property CDCover : TCDCover Read FCDCover Write SetCDCover;
    Property Config : TCDCoverConfiguration Read FConfig Write FConfig;
  end; 

var
  DiscEditorForm: TDiscEditorForm;

implementation

ResourceString
  sDiscCount = '%d discs';
  SNewTrack = 'Track no. %d';
  SNewDisc  = 'Disc no. %d';

{ TDiscEditorForm }

procedure TDiscEditorForm.AAddTrackExecute(Sender: TObject);

begin
  AddTrack;
end;

procedure TDiscEditorForm.AAddTrackUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=Assigned(FCurrentDisc);
end;

procedure TDiscEditorForm.ADeleteDiscExecute(Sender: TObject);
begin
  DeleteDisc;
end;

procedure TDiscEditorForm.ADeleteDiscUpdate(Sender: TObject);
begin
  (Sender as Taction).Enabled:=Assigned(FCurrentDisc);
end;

procedure TDiscEditorForm.ADeleteTrackExecute(Sender: TObject);
begin
  DeleteTrack
end;

procedure TDiscEditorForm.ADeleteTrackUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=Assigned(FCurrentTrack);
end;

procedure TDiscEditorForm.AScanDiscExecute(Sender: TObject);
begin
  ScanDisc;
end;

procedure TDiscEditorForm.ATrackDownExecute(Sender: TObject);
begin
  MoveTrack(1)
end;

procedure TDiscEditorForm.ATrackDownUpdate(Sender: TObject);
begin
  // If currenttrack is assigned, so is currentdisc
  (Sender as TAction).Enabled:=Assigned(FCurrentTrack) and
                              (FCurrentTrack.Index<FCurrentDisc.Tracks.Count-1);
end;

procedure TDiscEditorForm.ATrackUpExecute(Sender: TObject);

begin
  MoveTrack(-1);
end;

procedure TDiscEditorForm.MoveTrack(Step : Integer);

Var
  M1,M2 : TBaseMediator;
  N : TTreeNode;
  FPreviousTrack : TCDCoverTrack;

begin
  M1:=Nil;
  M2:=Nil;
  If Assigned(FCurrentTrackNode) then
    M1:=TBaseMediator(FCurrentTrackNode.Data);
  If (Step=-1) then
    N:=FCurrentTrackNode.GetPrevSibling
  else If (Step=1) then
    N:=FCurrentTrackNode.GetNextSibling;
  If (N<>Nil) then
    M2:=TBaseMediator(N.Data);
  If (M1=Nil) or (M2=Nil) then
    Exit;
  FPreviousTrack:=TCDCoverTracks(FCurrentTrack.Collection)[FCurrentTrack.Index+Step];
  FCurrentTrack.Index:=FCurrentTrack.Index+Step;
  If Assigned(M2) then
    M2.Subject:=FCurrentTrack;
  If Assigned(M1) then
    M1.Subject:=FPreviousTrack;
  If Assigned(N) then
    TVDiscs.Selected:=N;
end;

procedure TDiscEditorForm.ATrackUpUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=Assigned(FCurrentTrack) and (FCurrentTrack.Index>0);
end;

Function TDiscEditorForm.FindNodeForDisc(D : TCDDisc) : TTreeNode;

Var
  i : integer;

begin
  Result:=Nil;
  I:=0;
  While (Result=Nil) and (I<TVDiscs.Items.Count-1) do
    begin
    Result:=TVDiscs.Items[I];
    If Not (Assigned(Result.Data) and
            (TObject(Result.Data) is TBaseMediator) and
            (TBaseMediator(Result.Data).Subject=D)) then
    Inc(I);
    end;
end;

procedure TDiscEditorForm.ScanDisc;
Var
  D : TCDDisc;
  Q : TCDDBQuery;
  N : TTreeNode;
  I : Integer;

begin
  D:=AddDisc;
  Q:=TCDDBQuery.Create(Self);
  try
    Q.UseCache:=Config.CDDBUseCache;
    Q.CacheDir:=Config.CDDBCacheDir;
    Q.User:=Config.CDDBUSer;
    Q.HostURL:=Config.CDDBHostURL;
    Q.AppName:='CDCover';
    if not Q.QueryCD(Config.CDDevice,D) then
      DeleteDisc
    else
      begin
      If (FCurrentDisc=D) then
        N:=FCurrentDiscNode
      else
        N:=FindNodeForDisc(D);
      If Assigned(N) then
        begin
        For I:=0 to D.Tracks.Count-1 do
          AddTrackToTree(N,D.Tracks[i]);
        N.Expand(False);
        end;
      end;
  finally
    Q.Free;
  end;
end;

procedure TDiscEditorForm.FormCreate(Sender: TObject);
begin
  FTitleMediator:=TEditMediator.Create(Self);
  FTitleMediator.Control:=ETitle;
  FTitleMediator.SubjectPropertyName:='Title';
  FTitleMediator.active:=True;
  FPerformerMediator:=TEditMediator.Create(Self);
  FPerformerMediator.Control:=EPerformer;
  FPerformerMediator.SubjectPropertyName:='Performer';
  FPerformerMediator.Active:=True;
  FDurationMediator:=TTimeEditMediator.Create(Self);
  FDurationMediator.Control:=EDuration;
  FDurationMediator.Active:=True;
  FDurationMediator.SubjectPropertyName:='Duration';
  FCommentMediator:=TMemoMediator.Create(Self);
  FCommentMediator.Control:=MComment;
  FCommentMediator.SubjectPropertyName:='Extra';
  FCommentMediator.Active:=True;
//  FDurationMediator.ViewPropertyName:='Text';
end;

procedure TDiscEditorForm.AAddDiscExecute(Sender: TObject);

begin
  AddDisc;
end;

procedure TDiscEditorForm.TVDiscsSelectionChanged(Sender: TObject);

Var
  N : TTreeNode;
  M : TBaseMediator;

begin
  N:=TVDiscs.Selected;
  if (N=Nil) or (N=TVDiscs.Items[0]) then
    DisplayRoot
  else If TObject(N.Data) is TBaseMediator then
    begin
    M:=TBaseMediator(N.Data);
    If (M.Subject is TCDCoverTrack) then
      DisplayTrack(N,M.Subject as TCDCoverTrack)
    else If M.Subject is TCDDisc then
      DisplayDisc(N,M.Subject as TCDDisc);
    end;
end;

procedure TDiscEditorForm.DisplayRoot;

begin
  FDurationMediator.Subject:=Nil;
  FTitleMediator.Subject:=Nil;
  FPerformerMediator.Subject:=Nil;
  FCommentMediator.Subject:=Nil;
  FCurrentDiscNode:=Nil;
  FCurrentDisc:=Nil;
  FCurrentTrackNode:=Nil;
  FCurrentTrack:=Nil;
end;

procedure TDiscEditorForm.DisplayDisc(Node : TTreeNode; ADisc : TCDDisc);

begin
  FCurrentDiscNode:=Node;
  FCurrentDisc:=ADisc;
  FCurrentTrackNode:=Nil;
  FCurrentTrack:=Nil;
  FTitleMediator.Subject:=FCurrentDisc;
  FPerformerMediator.Subject:=FCurrentDisc;
  FCommentMediator.Subject:=FCurrentDisc;
  FDurationMediator.Subject:=Nil;
end;

procedure TDiscEditorForm.DisplayTrack(Node : TTreeNode; ATrack : TCDCoverTrack);

begin
  FCurrentTrackNode:=Node;
  FCurrentTrack:=ATrack;
  FCurrentDiscNode:=Node.Parent;
  FCurrentDisc:=TBaseMediator(FCurrentDiscNode.Data).Subject as TCDDisc;
  FTitleMediator.Subject:=FCurrentTrack;
  FPerformerMediator.Subject:=FCurrentTrack;
  FDurationMediator.Subject:=FCurrentTrack;
  FCommentMediator.Subject:=FCurrentTrack;
end;


procedure TDiscEditorForm.SetCDCover(const AValue: TCDCover);
begin
  FCDCover:=AValue;
  DisplayCover;
end;

procedure TDiscEditorForm.DisplayCover;

Var
  I,J : integer;
  D : TCDDisc;
  N : TTreeNode;

begin
  TVDiscs.Items[0].Text:=Format(SDiscCount,[FCDCover.DiscCount]);
  For I:=0 to FCDCover.DiscCount-1 do
    begin
    D:=FCDCover.Disc[i];
    N:=AddDiscToTree(D);
    For J:=0 to D.Tracks.Count-1 do
      AddTrackToTree(N,D.Tracks[J]);
    end;
  TVDiscs.Selected:=TVDiscs.Items[0];
  TVDiscs.Items[0].Expand(False);
end;

function TDiscEditorForm.AddTrack: TCDCoverTrack;

Var
  T : String;

begin
  Result:=Nil;
  If Not Assigned(CurrentDisc) then
    Exit;
  T:=Format(SNewTrack,[CurrentDisc.Tracks.Count+1]);
  Result:=CurrentDisc.Tracks.AddTrack(T);
  TVDiscs.Selected:=AddTrackToTree(FCurrentDiscNode,Result);
end;

function TDiscEditorForm.AddDiscToTree(ADisc : TCDDisc) : TTreeNode;

Var
  M : TTreeNodeMediator;

begin
  Result:=TVDiscs.Items.AddChild(TVDiscs.Items[0],ADisc.Caption);
  M:=TTreeNodeMediator.Create(Self);
  M.SubjectPropertyName:='Caption';
  M.Subject:=ADisc;
  M.TreeNode:=Result;
  M.Active:=True;
end;

function TDiscEditorForm.AddTrackToTree(AParent : TTreeNode; ATrack : TCDCoverTrack) : TTreeNode;

Var
  M : TTreeNodeMediator;

begin
  Result:=TVDiscs.Items.AddChild(AParent,ATrack.Caption);
  M:=TTreeNodeMediator.Create(Self);
  M.SubjectPropertyName:='Caption';
  M.Subject:=ATrack;
  M.TreeNode:=Result;
  M.Active:=True;
end;

function TDiscEditorForm.AddDisc: TCDDisc;

Var
  T : String;

begin
  T:=Format(SNewDisc,[CDCover.DiscCount+1]);
  Result:=CDCover.AddDisc;
  Result.Title:=T;
  TVDiscs.Items[0].Text:=Format(sDiscCount,[CDCover.DiscCount]);
  TVDiscs.Selected:=AddDiscToTree(Result);
end;

procedure TDiscEditorForm.DeleteDisc;

Var
  N : TTreeNode;

begin
  If Assigned(FCurrentDiscNode) and Assigned(FCurrentDiscNode.Data) then
    TBaseMediator(FCurrentDiscNode.Data).Free;
  N:=FCurrentDiscNode.GetNextSibling;
  If (N=nil) then
    N:=FCurrentDiscNode.GetPrevSibling;
  FreeAndNil(FCurrentDisc);
  FreeAndNil(FCurrentDiscNode);
  TVDiscs.Selected:=N;
end;

procedure TDiscEditorForm.DeleteTrack;

Var
  N : TTreeNode;

begin
  If Assigned(FCurrentTrackNode) and Assigned(FCurrentTrackNode.Data) then
    TBaseMediator(FCurrentTrackNode.Data).Free;
  N:=FCurrentTrackNode.GetNextSibling;
  If (N=nil) then
    begin
    N:=FCurrentTrackNode.GetPrevSibling;
    If N=Nil then
      N:=FCurrentTrackNode.Parent;
    end;
  FreeAndNil(FCurrentTrack);
  FreeAndNil(FCurrentTrackNode);
  TVDiscs.Selected:=N;
end;

{ TTreeNodeMediator }

procedure TTreeNodeMediator.SetTreeNode(const AValue: TTreeNode);
begin
  if FTreeNode=AValue then exit;
  FTreeNode:=AValue;
  If Assigned(FTreeNode) then
    FTreeNode.Data:=Self;
  ViewChanged;
end;

class function TTreeNodeMediator.PublishedViewProperty: Boolean;
begin
  Result:=False;
end;

function TTreeNodeMediator.GetView: TObject;
begin
  Result:=FTreeNode;
end;

procedure TTreeNodeMediator.DoViewToObject;
begin
  SetStrProp(Subject,SubjectPropertyName,FTreeNode.Text);
end;

procedure TTreeNodeMediator.DoObjectToView;
begin
  FTreeNode.Text:=GetStrProp(Subject,SubjectPropertyName);
end;

constructor TTreeNodeMediator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ViewPropertyName:='Text';
end;



{ TTimeEditMediator }

procedure TTimeEditMediator.DoViewToObject;

Var
  T : TDateTime;

begin
  If TryStrToTime(Control.Text,T) then
    SetFloatprop(Subject,SubjectPropertyName,T);
end;

procedure TTimeEditMediator.DoObjectToView;

begin
  Control.Text:=TimeToStr(GetFloatprop(Subject,SubjectPropertyName));
end;

initialization
  {$I frmdisceditor.lrs}

end.

