unit frmmain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  fpobserver, StdCtrls, cdcovers, Menus, ActnList, ComCtrls, covereditor,
  coverconfig, lclmediators;

type

  { TObjectMenuItem }

  TObjectMenuItem = Class (TMenuItem)
  private
    FOC: TCoverObjectClass;
  Public
    Property ObjectClass : TCoverObjectClass Read FOC Write FOC;
  end;

  { TMainForm }

  TMainForm = class(TForm)
    AClose: TAction;
    AAddPage: TAction;
    AAlign: TAction;
    AAlignHorizontalCenters: TAction;
    AAlignRight: TAction;
    AAlignTop: TAction;
    AAlignVerticalCenters: TAction;
    AAlignBottoms: TAction;
    AAlignLeft: TAction;
    AAddLabel: TAction;
    ACopy: TAction;
    AUndo: TAction;
    APaste: TAction;
    ACut: TAction;
    AEditCover: TAction;
    AResize: TAction;
    APrintSelected: TAction;
    APrintCurrentPage: TAction;
    AEditPreferences: TAction;
    AEditDiscs: TAction;
    AQuit: TAction;
    ASaveAs: TAction;
    ASave: TAction;
    AOpen: TAction;
    ANew: TAction;
    ALCover: TActionList;
    ILCover: TImageList;
    LVRecent: TListView;
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MEdit: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MIUndo: TMenuItem;
    MICut: TMenuItem;
    MIpaste: TMenuItem;
    MISEP10: TMenuItem;
    MICopy: TMenuItem;
    MIEditCover: TMenuItem;
    MIAlign: TMenuItem;
    MISep3: TMenuItem;
    MIAddObject: TMenuItem;
    MIAddPAge: TMenuItem;
    MISep: TMenuItem;
    MISep1: TMenuItem;
    MICover: TMenuItem;
    MIQuit: TMenuItem;
    MIClose: TMenuItem;
    MISep2: TMenuItem;
    MISaveAs: TMenuItem;
    MISave: TMenuItem;
    MIOpen: TMenuItem;
    MINew: TMenuItem;
    MFile: TMenuItem;
    ODCover: TOpenDialog;
    PCCovers: TPageControl;
    SDCover: TSaveDialog;
    TBCover: TToolBar;
    TBNew: TToolButton;
    TBAddLabel: TToolButton;
    TBPaste: TToolButton;
    TBOpen: TToolButton;
    TBSave: TToolButton;
    ToolButton4: TToolButton;
    TBAlignLeft: TToolButton;
    TBAlignCenter: TToolButton;
    TBAlignRight: TToolButton;
    ToolButton5: TToolButton;
    TBAlignTop: TToolButton;
    TBAlignVCenter: TToolButton;
    TBAlignBottom: TToolButton;
    ToolButton6: TToolButton;
    TBCopy: TToolButton;
    TBCut: TToolButton;
    ToolButton9: TToolButton;
    TSRecent: TTabSheet;
    procedure AAddLabelExecute(Sender: TObject);
    procedure AAddLabelUpdate(Sender: TObject);
    procedure AAddPageExecute(Sender: TObject);
    procedure AAddPageUpdate(Sender: TObject);
    procedure AAlignExecute(Sender: TObject);
    procedure AAlignUpdate(Sender: TObject);
    procedure ACloseExecute(Sender: TObject);
    procedure ACloseUpdate(Sender: TObject);
    procedure ACopyExecute(Sender: TObject);
    procedure ACopyUpdate(Sender: TObject);
    procedure ACutExecute(Sender: TObject);
    procedure AEditCoverExecute(Sender: TObject);
    procedure AEditCoverUpdate(Sender: TObject);
    procedure AEditDiscsExecute(Sender: TObject);
    procedure AEditDiscsUpdate(Sender: TObject);
    procedure AEditPreferencesExecute(Sender: TObject);
    procedure AEditPreferencesUpdate(Sender: TObject);
    procedure AOpenExecute(Sender: TObject);
    procedure APasteExecute(Sender: TObject);
    procedure APasteUpdate(Sender: TObject);
    procedure APrintCurrentPageExecute(Sender: TObject);
    procedure APrintCurrentPageUpdate(Sender: TObject);
    procedure AQuitExecute(Sender: TObject);
    procedure AResizeExecute(Sender: TObject);
    procedure AResizeUpdate(Sender: TObject);
    procedure ASaveAsExecute(Sender: TObject);
    procedure ASaveExecute(Sender: TObject);
    procedure ASaveUpdate(Sender: TObject);
    procedure AUndoExecute(Sender: TObject);
    procedure AUndoUpdate(Sender: TObject);
    procedure DoAlignAction(Sender: TObject);
    procedure LVRecentDblClick(Sender: TObject);
    procedure LVRecentKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState
      );
    procedure NewExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PCCoversChange(Sender: TObject);
    procedure UpdateAlignAction(Sender: TObject);
  private
    { private declarations }
    FConfig : TCDCoverConfiguration;
    FCurrentCover : TCDCover;
    FCurrentEditor : TCoverEditor;
    FRecentFiles : TRecentFiles;
    FRecentFilesMediator : TListViewMediator;
    function AddPages(E: TCoverEditor; PageTypes: TPageTypes): TCoverPage;
    procedure AddToRecentFiles(const AFileName: String);
    procedure AlignSelection;
    function CheckSave(AEditor: TCoverEditor): Boolean;
    function GetHaveEditor: Boolean;
    function HaveMultiSelection: Boolean;
    function HaveSelection: Boolean;
    procedure PrintCurrentPage;
    Procedure SetupObjectsMenu;
    Procedure DoObjectMenuClick(Sender : TObject);
    function SetupPrinter: Boolean;
  public
    { public declarations }
    Function NewCover(PageTypes : TPageTypes) : TCoverEditor;
    procedure SaveFile;
    procedure SaveFileAs;
    procedure OpenFile(AFileName: String);
    Property CurrentCover : TCDCover Read FCurrentCover;
    Property CurrentEditor : TCoverEditor Read FCurrentEditor;
    Property HaveEditor : Boolean Read GetHaveEditor;
  end;

var
  MainForm: TMainForm;

implementation

uses frmAdjustSize, typinfo, frmAlign, frmconfig, frmDiscEditor,
     frmnewcover, PrintersDlgs, Printers, frmaddpage;


resourcestring
  SModifiedSave = 'The CD-Cover design is modified.'+sLineBreak+
                  'Do you want to save the design ?';
{ TMainForm }

procedure TMainForm.FormCreate(Sender: TObject);
begin
  FRecentFiles:=TRecentFiles.CreateObserved(TRecentFile);
  FRecentFiles.LoadFromFile(GetAppConfigFile(False));
  FConfig:=TCDCoverConfiguration.Create(Self);
  FConfig.LoadConfig;
  FRecentFilesMediator:=TListViewMediator.Create(Self);
  FRecentFIlesMediator.Fieldsinfo.AddFieldInfo('Filename',200);
  FRecentFIlesMediator.Fieldsinfo.AddFieldInfo('Title',120);
  FRecentFIlesMediator.Fieldsinfo.AddFieldInfo('Performer',120);
  FRecentFilesMediator.Control:=LVRecent;
  FRecentFilesMediator.Subject:=FRecentFiles;
  FRecentFilesMediator.Active:=True;
  SetupObjectsMenu;
  TBCover.Images:=ILCover;
end;


procedure TMainForm.NewExecute(Sender: TObject);
begin
  With TNewCoverForm.Create(Self) do
    try
      PageTypes:=[ptBookletFront];
      If (ShowModal=mrOK) then
        NewCover(PageTypes);
    finally
      Free;
    end;
end;

procedure TMainForm.AEditDiscsUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=Assigned(FCurrentCover);
end;

procedure TMainForm.AEditPreferencesExecute(Sender: TObject);
begin
  With TConfigurationForm.Create(Self) do
    try
      Config:=FConfig;
      if (ShowModal=mrOK) then
        FConfig.SaveConfig;
    finally
      Free;
    end;
end;

procedure TMainForm.AEditPreferencesUpdate(Sender: TObject);
begin
  (sender as TAction).Enabled:=True;
end;

procedure TMainForm.AOpenExecute(Sender: TObject);
begin
  With ODCover do
    If Execute then
      OpenFile(FileName);
end;

procedure TMainForm.APasteExecute(Sender: TObject);
begin
  CurrentEditor.PasteSelectionFromClipboard;
end;

procedure TMainForm.APasteUpdate(Sender: TObject);
begin
{
  (Sender as TAction).Enabled:=HaveEditor and
     CurrentEditor.ClipboardContentOK;
}
end;

procedure TMainForm.APrintCurrentPageExecute(Sender: TObject);
begin
  PrintCurrentPage;
end;

Function TMainForm.SetupPrinter : Boolean;

Var
  D : TPrintDialog;

begin
  D:=TPrintDialog.Create(Self);
  Try
    Result:=D.Execute;
  Finally
    D.Free;
  end;
end;


procedure TMainForm.PrintCurrentPage;

Var
  O : TPoint;
  P : TCoverPage;

begin
  If Not SetupPrinter then exit;
  Printer.Title := 'Cover '+FCurrentEditor.FileName;
  Printer.BeginDoc;
  try
    O.X:=ToPoints(2,Printer.XDPI);
    O.Y:=ToPoints(2,Printer.XDPI);
    P:=FCurrentEditor.Currentpage;
    P.DrawPage(Printer.Canvas,O,Printer.YDPI);
  Finally
    Printer.EndDoc;
  end;
end;

procedure TMainForm.APrintCurrentPageUpdate(Sender: TObject);

begin
  (Sender as TAction).Enabled:=Assigned(FCurrentCover) and
                       Assigned(FCurrentEditor.Currentpage);
end;

procedure TMainForm.AddToRecentFiles(Const AFileName : String);

Var
  R : TRecentFile;

begin
  If FRecentFiles.IndexOfFileName(AFileName)=-1 then
    begin
    R:=FRecentFiles.Add as TRecentFile;
    R.FileName:=ExpandFilename(AFileName);
    end;
end;

procedure TMainForm.SetupObjectsMenu;

  Function AddItem(AClass : TCoverObjectClass; ACaption : String) : TObjectMenuItem;

  begin
    Result:=TObjectMenuItem.Create(Self);
    Result.Caption:=ACaption;
    Result.ObjectClass:=AClass;
    Result.OnClick:=@DoObjectMenuClick;
    MIAddObject.Add(Result);
  end;

begin
  AddItem(TLabelObject,'Label');
  AddItem(TShapeObject,'Shape');
  AddItem(TTrackList,'Track list');
  AddItem(TImageObject,'Image');
end;

procedure TMainForm.DoObjectMenuClick(Sender: TObject);
begin
  If HaveEditor and (Sender is TObjectMenuItem) then
   FCurrentEditor.AddObject((Sender as TObjectMenuItem).ObjectClass);
end;

procedure TMainForm.OpenFile(AFileName : String);

Var
  E : TCoverEditor;
  C : TCDCover;


begin
  If fileexists(AFileName) then
    try
      E:=NewCover([]);
      E.LoadFromFile(AFileName);
      AddToRecentFiles(AFileName);
    except
      C.Free;
      Raise;
    end;
end;

procedure TMainForm.AQuitExecute(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.AResizeExecute(Sender: TObject);
begin
  If Not HaveEditor then
    Exit;
  With TAdjustSizeForm.Create(Self) do
    try
      if ShowModal=mrOK then
        CurrentEditor.ResizeSelection(VOperation,VSize,
                                      HOperation,HSize);
    finally
      Free;
    end;

end;

Function TMainForm.HaveSelection : Boolean;

begin
 Result:=HaveEditor and (FCurrentEditor.SelectionCount>0);
end;

Function TMainForm.HaveMultiSelection : Boolean;
begin
  Result:=HaveEditor and (FCurrentEditor.SelectionCount>1);
end;

procedure TMainForm.AResizeUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=HaveSelection
end;

procedure TMainForm.ASaveAsExecute(Sender: TObject);
begin
  SaveFileAs;
end;

procedure TMainForm.ASaveExecute(Sender: TObject);
begin
  SaveFile;
end;

procedure TMainForm.SaveFile;

begin
  If (FCurrentEditor.FileName='') then
    SaveFileAs
  else
    FCurrentEditor.SaveToFile('');
end;

procedure TMainForm.SaveFileAs;

begin
  With SDCover do
    If Execute then
      begin
      FCurrentEditor.SaveToFile(FileName);
      AddToRecentFiles(FileName);
      end;
end;


procedure TMainForm.ASaveUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=HaveEditor;
end;

procedure TMainForm.AUndoExecute(Sender: TObject);
begin
  CurrentEditor.UndoLastAction;
end;

procedure TMainForm.AUndoUpdate(Sender: TObject);

Var
  S : String;

begin
  if HaveEditor then
    S:=CurrentEditor.CurrentUndo;
  With (Sender as TAction) do
    begin
    Enabled:=(S<>'');
    if Enabled then
      begin
      Caption:='Undo '+S;
      Hint:='Undo '+S;
      end
    else
      begin
      Caption:='Undo';
      Hint:='No Undo action available';
      end;
    end;
end;

procedure TMainForm.DoAlignAction(Sender: TObject);

Var
  HA : THAlignAction;
  VA : TVAlignAction;


begin
  HA:=haNone;
  VA:=vaNone;
  Case (sender as TAction).Tag of
    1 : HA:=haLeft;
    2 : HA:=haCenter;
    3 : HA:=haRight;
    4 : VA:=vaTop;
    5 : VA:=vaCenter;
    6 : VA:=vaBottom;
  end;
  FCurrentEditor.AlignSelection(HA,VA);
end;

procedure TMainForm.LVRecentDblClick(Sender: TObject);

Var
  R:TRecentFile;

begin
  R:=TRecentFile(FRecentFilesMediator.SelectedObject);
  If Assigned(R) then
    OpenFile(R.FileName);
end;

procedure TMainForm.LVRecentKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin

end;

procedure TMainForm.AEditDiscsExecute(Sender: TObject);
begin
  With TDiscEditorForm.Create(Self) do
    try
      CDCover:=FCurrentCover;
      Config:=FConfig;
      ShowModal;
    finally
      Free;
    end;
end;

procedure TMainForm.AAddPageExecute(Sender: TObject);

Var
  E : TCoverEditor;
  P : TCoverPage;

begin
  E:=CurrentEditor;
  If E=Nil then
    Exit;
  With TAddPagesForm.Create(Self) do
    try
      CDCover:=CurrentCover;
      If ShowModal=mrOK then
        begin
        P:=AddPages(E,PageTypes);
        If Assigned(P) then
          E.Currentpage:=P;
        end;
    finally
      Free;
    end;
end;

procedure TMainForm.AAddLabelUpdate(Sender: TObject);
begin
  (Sender as Taction).Enabled:=HaveEditor;
end;

procedure TMainForm.AAddLabelExecute(Sender: TObject);
begin
  FCurrentEditor.AddObject(TLabelObject);
end;

procedure TMainForm.AAddPageUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=HaveEditor;
end;

procedure TMainForm.AAlignExecute(Sender: TObject);
begin
  AlignSelection;
end;

procedure TMainForm.AlignSelection;

begin
  If Not HaveEditor then
    Exit;
  With TAlignForm.Create(Self) do
    try
      if (ShowModal=mrOK) then
        FCurrentEditor.AlignSelection(Horizontal,
                                      Vertical);
    finally
      Free;
    end;
end;

procedure TMainForm.AAlignUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=HaveSelection;
end;

procedure TMainForm.ACloseExecute(Sender: TObject);
begin
  If HaveEditor then
    If CheckSave(FCurrentEditor) then
      FCurrentEditor.Free;
end;

Function TMainForm.CheckSave(AEditor : TCoverEditor) : Boolean;

begin
  Result:=Not AEditor.Modified;
  If Not Result then
    begin
    Case MessageDLG(SModifiedSave,mtInformation,[mbYes,mbNo,mbCancel],0) of
      mrYes : begin
              AEditor.SaveToFile(AEditor.FileName);
              Result:=True;
              end;
      MrNo : Result:=True;
    end;
    end;
end;

function TMainForm.GetHaveEditor: Boolean;
begin
  Result:=Assigned(FCurrentEditor)
end;


procedure TMainForm.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FrecentFilesMediator);
  FRecentFiles.SaveToFile(GetAppConfigFile(False));
  FreeAndNil(FRecentFiles);
end;

procedure TMainForm.PCCoversChange(Sender: TObject);
begin
  If PCCovers.ActivePage is TCoverEditor then
    begin
    FCurrentEditor:=PCCovers.ActivePage as TCoverEditor;
    FCurrentCover:=FCurrentEditor.CDCover;
    end
  else
    begin
    FCurrentEditor:=Nil;
    FCurrentCover:=Nil;
    end;
end;

procedure TMainForm.UpdateAlignAction(Sender: TObject);
begin
  (Sender as TAction).Enabled:=HaveMultiSelection;
end;

Function TMainForm.NewCover(PageTypes : TPageTypes) : TCoverEditor;

  Function NewFileName : String;

  Var
    N,I : Integer;
    FN :String;
    B : Boolean;

  begin
    N:=0;
    Repeat
      Inc(N);
      Result:='Cover'+IntToStr(N);
      I:=1;
      B:=False;
      While (I<PCCovers.PageCount) and Not B do
        begin
        If PCCovers.Pages[i] is TCoverEditor then
          begin
          FN:=TCoverEditor(PCCovers.Pages[i]).Caption;
          B:=CompareFileNames(FN,Result)=0;
          end;
        Inc(I);
        end;
    until Not B;
  end;

Var
  E : TCoverEditor;
  P : TCoverPage;

begin
  E:=TCoverEditor.Create(Self);
  E.Parent:=PCCovers;
  E.CDCover:=TCDCover.Create(E);
  E.CDCover.Name:='CDCover';
  E.Caption:=NewFileName;
  P:=AddPages(E,PageTypes);
  If Assigned(P) then
    E.Currentpage:=P;
//  ,ptBookletBack,ptInlay,ptDiscLabel
  PCCovers.ActivePage:=E;
  PCCoversChange(PCCovers);
  Result:=E;
end;

Function TMainForm.AddPages(E : TCoverEditor;PageTypes : TPageTypes) : TCoverPage;

Var
  T : TPageType;

begin
  Result:=Nil;
  for T:=Low(TPageType) to High(TPageType) do
    If T in PageTypes then
      If Result=Nil then
        Result:=E.AddPage(T)
      else
        E.AddPage(T);
end;

procedure TMainForm.ACloseUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=HaveEditor;
end;

procedure TMainForm.ACopyExecute(Sender: TObject);
begin
  CurrentEditor.CopySelectionToClipboard(True);
end;

procedure TMainForm.ACopyUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=HaveEditor and (CurrentEditor.SelectionCount>0);
end;

procedure TMainForm.ACutExecute(Sender: TObject);
begin
  CurrentEditor.CopySelectionToClipBoard(True);
  CurrentEditor.DeleteSelection;
end;

procedure TMainForm.AEditCoverExecute(Sender: TObject);

Var
  I : Integer;
  R : TRecentFile;

begin
  if FCurrentEditor.EditProperties then
    begin
    I:=FRecentFiles.IndexOfFileName(FCurrentEditor.FileName);
    If (I<>-1) then
      begin
      R:=FRecentFiles[i];
      R.Performer:=FCurrentEditor.CDCover.Performer;
      R.Title:=FCurrentEditor.CDCover.Title;
      end;
    end;
end;

procedure TMainForm.AEditCoverUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=HaveEditor;
end;

initialization
  {$I frmmain.lrs}

end.

