{
  Purpose:
    Abstract mediating view and Mediator Factory. This allows you to use
    standard edit components and make them object-aware.  See the demo
    application for usage.

}

unit lclmediators;

{$mode objfpc}{$H+}

interface
uses
  Classes
  ,fpobserver
  ,Controls
  ,stdctrls
  ,extctrls
  ,comctrls
  ,spin
  ,EditBtn
  ,buttons
  ,Dialogs
  ;

Const
  DefDisableControl = True;

type

  { TControlMediator - Base for all control-related mediators }

  TControlMediator = Class(TBaseMediator)
  private
    FDisableControl: Boolean;
    procedure SetDisableControl(const AValue: Boolean);
  Protected
    // Call checkdisablecontrol
    procedure SetSubject(const AValue: TObject); override;
    // handles remove/freenotifications, and sets TheControl to AControl, calls viewchanged.
    Procedure SetControl(Var TheControl : TControl; Const AControl : TControl);
    // Checks if control must be disabled
    Procedure CheckDisableControl; virtual;
     // Calls setupcontrol
    Procedure ViewChanged; override;
    // Calls checkdisablecontrol. Override to do additional setup.
    procedure SetupControl; virtual;
  Public
    Constructor Create(AOwner : TComponent); override;
  Published
    // Disable control if Subject not set.
    Property DisableControl : Boolean Read FDisableControl Write SetDisableControl Default DefDisableControl;
  end;

  { TEditMediator - Base class to handle TCustomEdit controls }

  TEditMediator = class(TControlMediator)
  private
    FEdit : TCustomEdit;
    procedure SetEdit(const AValue: TCustomEdit);
  protected
    function GetView: TObject; override;
    procedure SetupControl; override;
    Procedure CheckDisableControl; override;
  public
    Constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);override;
    class function ViewClass: TClass; override;
  Published
    property Control: TCustomEdit read FEdit write SetEdit;
  end;

  { TFileNameEditMediator }

  TFileNameEditMediator = class(TEditMediator)
  Public
    Constructor Create(AOwner : TComponent); override;
    class function ViewClass: TClass; override;
  end;

  { TDirectoryEditMediator }

  TDirectoryEditMediator = class(TEditMediator)
  Public
    Constructor Create(AOwner : TComponent); override;
    class function ViewClass: TClass; override;
  end;

  { TDateEditMediator - Base class to handle TCustomEdit controls }

  TDateEditMediator = class(TControlMediator)
  private
    FEdit : TDateEdit;
    procedure SetEdit(const AValue: TDateEdit);
  protected
    function GetView: TObject; override;
    procedure SetupControl; override;
    Procedure CheckDisableControl; override;
  public
    Constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);override;
    class function ViewClass: TClass; override;
  Published
    property Control: TDateEdit read FEdit write SetEdit;
  end;


  { TCheckboxMediator }

  TCheckboxMediator = class(TControlMediator)
  private
    FCheckBox : TCheckBox;
    procedure SetCheckBox(const AValue: TCheckBox);
  Protected
    function GetView: TObject; override;
    procedure SetupControl; override;
  public
    Constructor Create(AOwner : TComponent); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    class function ViewClass: TClass; override;
  Published
    property Control: TCheckBox read FCheckBox write SetCheckBox;
  end;

  { TRadioGroupMediator }

  TRadioGroupMediator = class(TControlMediator)
  private
    FRadioGroup : TRadioGroup;
    procedure SetRadioGroup(const AValue: TRadioGroup);
  Protected
    function GetView: TObject; override;
    procedure SetupControl; override;
    procedure DoObjectToView; override;
    procedure DoViewToObject; override;
  public
    Constructor Create(AOwner : TComponent); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    class function ViewClass: TClass; override;
  Published
    property Control: TRadioGroup read FRadioGroup write SetRadioGroup;
  end;

  { TColorButtonMediator }

  TColorButtonMediator = class(TControlMediator)
  private
    FColorButton: TColorButton;
    procedure SetColorButton(const AValue: TColorButton);
  Protected
    function GetView: TObject; override;
    procedure SetupControl; override;
  public
    Constructor Create(AOwner : TComponent); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    class function ViewClass: TClass; override;
  Published
    property Control: TColorButton read FColorButton write SetColorButton;
  end;


  { Base class to handle TCustomLabel controls }

  { TLabelMediator }

  TLabelMediator = class(TControlMediator)
  private
    FLabel: TCustomLabel;
    procedure SetLabel(const AValue: TCustomLabel);
  protected
    function    GetView : TObject; override;
  public
    Constructor Create(AOwner : TComponent); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    class function ViewClass: TClass; override;
  Published
    property Control : TCustomLabel read FLabel write SetLabel;
  end;


  { TSpinEditMediator - Base class to handle TSpinEdit controls }

  TSpinEditMediator = class(TControlMediator)
  private
    FSpinEdit: TSpinEdit;
    procedure SetSpinEdit(const AValue: TSpinEdit);
  protected
    function    GetView : TObject; override;
    procedure SetupControl; override;
  public
    Constructor Create(AOwner : TComponent); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    class function ViewClass: TClass; override;
  Published
    property Control: TSpinEdit read FSpinEdit write SetSpinEdit;
  end;

  { TSpinEditMediator - Base class to handle TSpinEdit controls }

  { TFloatSpinEditMediator }

  TFloatSpinEditMediator = class(TControlMediator)
  private
    FSpinEdit: TFloatSpinEdit;
    procedure SetSpinEdit(const AValue: TFloatSpinEdit);
  protected
    function    GetView : TObject; override;
    procedure SetupControl; override;
  public
    Constructor Create(AOwner : TComponent); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    class function ViewClass: TClass; override;
  Published
    property Control: TFloatSpinEdit read FSpinEdit write SetSpinEdit;
  end;


  { TTrackBarMediator - Base class to handle TTrackBar controls }

  TTrackBarMediator = class(TControlMediator)
  private
    FTrackBar: TTrackBar;
    procedure SetTrackBar(const AValue: TTrackBar);
  protected
    function  GetView : TObject; override;
    procedure SetupControl; override;
  public
    Constructor Create(AOwner : TComponent); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    class function ViewClass: TClass; override;
  Published
    property Control : TTrackBar read FTrackBar write SetTrackBar;
  end;

  { TCustomComboBoxMediator - Base class to handle TComboBox controls}

  TCustomComboBoxMediator = class(TControlMediator)
  private
    FComboBox : TComboBox;
    FUseItemIndex: Boolean;
    procedure SetComboBox(const AValue: TComboBox);
    procedure SetUseItemIndex(const AValue: Boolean);
  protected
    procedure SetupControl; override;
    procedure CheckViewPropertyName;
    Function  GetView : TObject; override;
    Function DefaultUseItemindex : Boolean; virtual;
    Property UseItemIndex : Boolean Read FUseItemIndex Write SetUseItemIndex;
  public
    Constructor Create(AOwner : TComponent); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    class function ViewClass: TClass; override;
  Published
    property Control: TComboBox read FComboBox write SetComboBox;
  end;

  { TComboBoxMediator - Publishes UseItemIndex }

  TComboBoxMediator = Class(TCustomComboBoxMediator)
  Published
    Property UseItemindex default false;
  end;

  { TItemComboBoxMediator - Always uses ItemIndex }

  TItemComboBoxMediator = Class(TCustomComboBoxMediator)
  protected
    Function DefaultUseItemindex : Boolean; override;
  end;

  { TObjectComboBoxMediator }

  TObjectComboBoxMediator = class(TControlMediator)
  private
    FComboBox  : TComboBox;
    FDisplayFieldName: String;
    Function GetDisplayFieldName: String;
    procedure SetComboBox(const AValue: TComboBox);
    procedure SetDisplayFieldName(const AValue: String);
  protected
    function GetView: TObject; override;
    Procedure   ValueListChanged; override;
    procedure   SetupControl; override;
    procedure   DoObjectToView; override;
    procedure   DoViewToObject; override;
  public
    Constructor Create(AOwner : TComponent); override;
    procedure   RefreshList; virtual;
  Published
    Property Control : TComboBox Read FComboBox Write SetComboBox;
    Property    DisplayFieldName : String Read GetDisplayFieldName Write SetDisplayFieldName;
  end;

  { TMemoMediator - Base class to handle TMemo controls }

  TMemoMediator = class(TControlMediator)
  private
    FMemo : TMemo;
    FStringsProperty: Boolean;
    procedure CheckPropertyType;
    procedure SetMemo(const AValue: TMemo);
  Protected
    function  GetView : TObject; override;
    procedure SetupControl; override;
    procedure SetSubject(const AValue: TObject); override;
    procedure SetSubjectPropertyName(const AValue: String); override;
    procedure DoObjectToView; override;
    procedure DoViewToObject; override;
    Property StringsProperty : Boolean Read FStringsProperty;
  public
    Constructor Create(AOwner : TComponent); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    class function ViewClass: TClass; override;
  Published
    property Control : TMemo read FMemo write SetMemo;
  end;

  { TSpeedButtonMediator - Set Down based on boolean value }

  TSpeedButtonMediator = Class(TControlMediator)
  private
    FButton: TSpeedbutton;
    procedure SetButton(const AValue: TSpeedbutton);
  Protected
    Procedure DoObjectToView; override;
    Procedure DoViewToObject; override;
    function  GetView : TObject; override;
    procedure SetupControl; override;
    Function GetBooleanValue : Boolean; virtual;
    Procedure SetBooleanValue(AValue : Boolean); virtual;
  Public
    Constructor Create(AOwner : TComponent); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    Property Control : TSpeedbutton Read FButton Write SetButton;
  end;

  { TImageMediator - Set Down based on boolean value }

  TImageMediator = Class(TControlMediator)
  private
    FCenteredProperty: String;
    FImage: TImage;
    FStretchedProperty: String;
    procedure SetCenteredProperty(const AValue: String);
    procedure SetImage(const AValue: TImage);
    procedure SetStretchedProperty(const AValue: String);
  Protected
    Procedure DoObjectToView; override;
    Procedure DoViewToObject; override;
    function  GetView : TObject; override;
    procedure SetupControl; override;
  Public
    Constructor Create(AOwner : TComponent); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    Property Control : TImage Read FImage Write SetImage;
    // Will be used in ObjectToView to center/stretch the image, if set.
    Property CenterPropertyName : String Read FCenteredProperty Write SetCenteredProperty;
    Property StretchPropertyName : String Read FStretchedProperty Write SetStretchedProperty;
  end;

  { TListViewItemMediator }

  TListViewItemMediator = Class(TColumnsListItemMediator)
  private
    FListItem: TListItem;
  Public
    Procedure ObjectToView; override;
    Property ListItem : TListItem Read FListItem;
  end;

  { TListViewMediator }

  TListViewMediator = Class(TColumnsListMediator)
  private
    FListView: TListView;
    procedure SetListView(const AValue: TListView);
  protected
    function GetSelectedObject: TObject; override;
    procedure SetSelectedObject(const AValue: TObject); override;
    Function  CreateItemMediator(AData: TObject; ARow : integer) : TListItemMediator; override;
    procedure DoDeleteItemMediator(AIndex : Integer; AMediator : TListItemMediator); override;
    function  GetView : TObject; override;
    procedure CreateColumns; override;
  Public
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    Property Control : TListView Read FListView Write SetListView;
  end;

// Registering generic mediators which can handle most cases by default.
// procedure RegisterFallBackMediators;



implementation
uses
  SysUtils
  ,TypInfo
  ,graphics
  ;

Resourcestring
//  cErrorListHasNotBeenAssigned   = 'List has not been assigned';
//  cErrorPropertyNotClass         = 'Property is not a class type!';
{
  cErrorAddingItemToCombobox     = 'Error adding list items to combobox ' +
                                   'Message: %s, Item Property Name: %s';
}
  SErrInvalidPropertyKind  = '%s: Cannot handle type of property: "%s"';
  SErrUnknownImageClass    = 'Unknown image class %s';
  SErrUnknownProperty      = '%s : %s is not a published property of class';

procedure RegisterFallBackMediators;
begin
{
  gMediatorManager.RegisterMediator(TEditMediator, TtiObject, [tkSString,tkAString,tkInteger,tkFloat]);
  gMediatorManager.RegisterMediator(TCheckboxMediator, TtiObject, [tkBool]);
  gMediatorManager.RegisterMediator(TMediatorComboboxView, TtiObject, [tkSString,tkAString]);
  gMediatorManager.RegisterMediator(TLabelMediator, TtiObject);
  gMediatorManager.RegisterMediator(TMediatorTrackBarView, TtiObject, [tkInteger]);
  gMediatorManager.RegisterMediator(TMediatorDynamicComboBoxView, TtiObject, [tkClass]);
  gMediatorManager.RegisterMediator(TMediatorMemoView, TtiObject, [tkSString,tkAString]);
  gMediatorManager.RegisterMediator(TMediatorSpinEditView, TtiObject, [tkInteger,tkFloat]);
}
end;


{ TEditMediator }

constructor TEditMediator.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  ViewPropertyName:='Text';
end;

procedure TEditMediator.SetEdit(const AValue: TCustomEdit);
begin
  SetControl(FEdit,AValue);
end;

function TEditMediator.GetView: TObject;
begin
  Result:=FEdit;
end;

procedure TEditMediator.SetupControl;
begin
  Inherited;
  FEdit.OnChange:=@ViewChangedHandler;
  FEdit.Clear;
end;

procedure TEditMediator.CheckDisableControl;
begin
  inherited CheckDisableControl;
  If Not FEdit.Enabled then
    FEdit.Clear;
end;

procedure TEditMediator.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  If (Operation=opRemove) and (AComponent=FEdit) then
    begin
    FEdit:=Nil;
    ViewChanged;
    end;
end;

destructor TEditMediator.Destroy;
begin
  if Assigned(FEdit) then
    FEdit.OnChange := nil;
  inherited Destroy;
end;

class function TEditMediator.ViewClass: TClass;
begin
  Result := TCustomEdit;
end;


{ TMediatorSpinEditView}

(*


{ TMediatorComboBoxView }

class function TMediatorComboBoxView.ComponentClass: TClass;
begin
  Result := TComboBox;
end;

function TMediatorComboBoxView.GetGUIControl: TComponent;
begin
  Result := FEditControl;
end;

procedure TMediatorComboBoxView.SetGUIControl(const AValue: TComponent);
begin
  FEditControl := AValue as TComboBox;
  inherited SetGUIControl(AValue);
end;

procedure TMediatorComboBoxView.SetupGUIandObject;
begin
  inherited SetupGUIandObject;
  if ObjectUpdateMoment in [ouOnChange,ouCustom] then
    FEditControl.OnChange := @DoOnChange
  else
    FEditControl.OnExit := @DoOnChange;
end;

procedure TMediatorComboBoxView.UpdateGuiValidStatus(pErrors: TtiObjectErrors);
var
  oError: TtiObjectError;
begin
  inherited UpdateGuiValidStatus(pErrors);

  oError := pErrors.FindByErrorProperty(FieldName);
  if oError <> nil then
  begin
    EditControl.Color  := clError;
    EditControl.Hint   := oError.ErrorMessage;
  end
  else
  begin
    EditControl.Color  := clWindow;
    EditControl.Hint   := '';
  end;
end;

constructor TMediatorComboBoxView.Create;
begin
  inherited Create;
  GuiFieldName := 'Text';
end;

procedure TMediatorComboBoxView.DoObjectToGui;
begin
  EditControl.ItemIndex :=
      EditControl.Items.IndexOf(Subject.PropValue[FieldName]);
end;


{ TMediatorMemoView }

class function TMediatorMemoView.ComponentClass: TClass;
begin
  Result := TMemo;
end;

procedure TMediatorMemoView.DoGuiToObject;
begin
  Subject.PropValue[FieldName] := EditControl.Lines.Text;
end;

procedure TMediatorMemoView.DoObjectToGui;
begin
  EditControl.Lines.Text := Subject.PropValue[FieldName];
end;

function TMediatorMemoView.GetGUIControl: TComponent;
begin
  Result:=FEditControl;
end;

procedure TMediatorMemoView.SetGUIControl(const AValue: TComponent);
begin
  FEditControl:=AValue as TMemo;
  inherited SetGUIControl(AValue);
end;

procedure TMediatorMemoView.SetupGUIandObject;
begin
  inherited;
  EditControl.Lines.Clear;
  if ObjectUpdateMoment in [ouOnChange,ouCustom] then
    FEditControl.OnChange := @DoOnChange
  else
    FEditControl.OnExit := @DoOnChange;
end;


{ TMediatorDynamicComboBoxView }

procedure TMediatorDynamicComboBoxView.SetListObject(const AValue: TtiObjectList);
begin
  inherited;
  InternalListRefresh;
  if Assigned(ValueList) then
    EditControl.Enabled := ValueList.Count > 0;
end;

procedure TMediatorDynamicComboBoxView.InternalListRefresh;
var
  lItems: TStrings;
  i: Integer;
begin
  lItems := EditControl.Items;
  lItems.Clear;
  EditControl.Text := '';

  if (ValueList = nil) or
     (ValueList.Count < 1) or
     (SameText(FieldName, EmptyStr)) then
    Exit; //==>

  try
    for i := 0 to ValueList.Count - 1 do
    begin
      lItems.AddObject(GetStrProp(ValueList.Items[i],DisplayFieldName),ValueList.Items[i]);
    end;
  except
    on E: Exception do
      RaiseMediatorError(cErrorAddingItemToCombobox,[E.message, FieldName]);
  end;

  ObjectToGui;
end;

function TMediatorDynamicComboBoxView.GetDisplayFieldName: String;
begin
  Result:=FDisplayFieldName;
  If (Result='') then
    Result:='Caption'; // Do not localize.
end;

procedure TMediatorDynamicComboBoxView.SetOnChangeActive(AValue: Boolean);
begin
  if AValue then
  begin
    if not UseInternalOnChange then
      EditControl.OnChange := FExternalOnChange
    else
      EditControl.OnChange := @DoOnChange;
  end
  else
  begin
    if not UseInternalOnChange then
      FExternalOnChange := EditControl.OnChange;
    EditControl.OnChange := nil;
  end;
end;

procedure TMediatorDynamicComboBoxView.SetupGUIandObject;
begin
  inherited SetupGUIandObject;

  if UseInternalOnChange then
    EditControl.OnChange := @DoOnChange; // default OnChange event handler

  {$Note As far as I can see, ValueList is always going to be nil here! - Graeme }
  if ValueList <> nil then
    EditControl.Enabled := (ValueList.Count > 0);
end;

procedure TMediatorDynamicComboBoxView.DoGuiToObject;
var
  lValue: TtiObject;
  lPropType: TTypeKind;
begin
  if not DataAndPropertyValid then
    Exit; //==>
  if EditControl.ItemIndex < 0 then
    Exit; //==>

  lValue := TtiObject(ValueList.Items[EditControl.ItemIndex]);

  lPropType := typinfo.PropType(Subject, FieldName);
  if lPropType = tkClass then
    typinfo.SetObjectProp(Subject, FieldName, lValue)
  else
    RaiseMediatorError(cErrorPropertyNotClass);
end;

procedure TMediatorDynamicComboBoxView.DoObjectToGui;
var
  i: Integer;
  lValue: TtiObject;
  lPropType: TTypeKind;
begin
  SetOnChangeActive(false);

  //  Set the index only (We're assuming the item is present in the list)
  EditControl.ItemIndex := -1;
  if Subject = nil then
    Exit; //==>

  if not Assigned(ValueList) then
    RaiseMediatorError(cErrorListHasNotBeenAssigned);

  lPropType := typinfo.PropType(Subject, FieldName);
  if lPropType = tkClass then
    lValue := TtiObject(typinfo.GetObjectProp(Subject, FieldName))
  else
    RaiseMediatorError(cErrorPropertyNotClass);

  for i := 0 to ValueList.Count - 1 do
    if ValueList.Items[i] = lValue then
    begin
      EditControl.ItemIndex := i;
      Break; //==>
    end;

  SetOnChangeActive(true);
end;

procedure TMediatorDynamicComboBoxView.RefreshList;
begin
  InternalListRefresh;
end;


*)



procedure TCheckboxMediator.SetCheckBox(const AValue: TCheckBox);
begin
  SetControl(FCheckBox,AValue);
end;

function TCheckboxMediator.GetView: TObject;
begin
  Result:=FCheckBox;
end;

procedure TCheckboxMediator.SetupControl;
begin
  inherited SetupControl;
  FCheckBox.OnClick := @ViewChangedHandler
end;

constructor TCheckboxMediator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ViewPropertyName:='Checked';
end;

procedure TCheckboxMediator.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  Inherited;
  If (Operation=opRemove) and (AComponent=FCheckBox) then
    begin
    FCheckBox:=Nil;
    ViewChanged;
    end;
end;

class function TCheckboxMediator.ViewClass: TClass;
begin
  Result:=inherited ViewClass;
end;


{ TLabelMediator }

procedure TLabelMediator.SetLabel(const AValue: TCustomLabel);
begin
  SetControl(Flabel,AValue);
end;

function TLabelMediator.GetView: TObject;
begin
  Result:=FLabel;
end;

constructor TLabelMediator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ViewPropertyName:='Caption';
  ReadOnly:=True;
end;

procedure TLabelMediator.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  If (Operation=opRemove) and (AComponent=Flabel) then
    begin
    FLabel:=Nil;
    ViewChanged;
    end;
end;

class function TLabelMediator.ViewClass: TClass;
begin
  Result:=TCustomLabel;
end;

{ TControlMediator }


procedure TControlMediator.SetDisableControl(const AValue: Boolean);
begin
  if FDisableControl=AValue then exit;
  FDisableControl:=AValue;
  CheckDisableControl;
end;

procedure TControlMediator.SetSubject(const AValue: TObject);
begin
  inherited SetSubject(AValue);
  CheckDisableControl;
end;

procedure TControlMediator.SetControl(var TheControl: TControl;
  const AControl: TControl);
begin
  if (TheControl=AControl) then exit;
  If Assigned(TheControl) then
    TheControl.RemoveFreeNotification(Self);
  TheControl:=AControl;
  If Assigned(TheControl) then
    TheControl.FreeNotification(Self);
  ViewChanged;
end;

procedure TControlMediator.CheckDisableControl;

Var
  V : TObject;

begin
  V:=View;
  If Assigned(V) and (V is TControl) then
    begin
    TControl(V).Enabled:=(Not FDisableControl) or Assigned(Subject);
    end;
end;

procedure TControlMediator.ViewChanged;
begin
  inherited ViewChanged;
  SetupControl;
end;

procedure TControlMediator.SetupControl;
begin
  CheckDisableControl;
end;

constructor TControlMediator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDisableControl:=True;
end;

{ TSpinEditMediator }

procedure TSpinEditMediator.SetSpinEdit(const AValue: TSpinEdit);
begin
  SetControl(FSpinEdit,AValue);
end;

function TSpinEditMediator.GetView: TObject;
begin
  Result:=FSpinEdit;
end;

procedure TSpinEditMediator.SetupControl;
begin
  inherited SetupControl;
  FSpinEdit.OnChange:=@ViewChangedHandler;
  FSpinEdit.Clear;
end;

constructor TSpinEditMediator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ViewPropertyName:='Value';
end;

procedure TSpinEditMediator.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  If (Operation=opRemove) and (AComponent=FSpinEdit) then
    begin
    FSpinEdit:=Nil;
    ViewChanged;
    end;
end;

class function TSpinEditMediator.ViewClass: TClass;
begin
  Result:=TSpinEdit;
end;

{ TTrackBarMediator }

procedure TTrackBarMediator.SetTrackBar(const AValue: TTrackBar);
begin
  SetControl(FTrackBar,AValue);
end;


function TTrackBarMediator.GetView: TObject;
begin
  Result:=FTrackBar;
end;

procedure TTrackBarMediator.SetupControl;
begin
  inherited SetupControl;
  FTrackBar.OnChange := @ViewChangedHandler;
  FTrackBar.Position:=0;
end;

constructor TTrackBarMediator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ViewPropertyName := 'Position';
end;

procedure TTrackBarMediator.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  If (Operation=opRemove) and (AComponent=FTrackBar) then
    begin
    FTrackBar:=Nil;
    ViewChanged;
    end;
end;

class function TTrackBarMediator.ViewClass: TClass;
begin
  Result:=TTrackBar;
end;

{ TCustomComboBoxMediator }

procedure TCustomComboBoxMediator.SetComboBox(const AValue: TComboBox);
begin
  SetControl(FComboBox,AValue);
end;

procedure TCustomComboBoxMediator.SetUseItemIndex(const AValue: Boolean);
begin
  if FUseItemIndex=AValue then exit;
  FUseItemIndex:=AValue;
  CheckViewPropertyName;
end;

function TCustomComboBoxMediator.GetView: TObject;
begin
  Result:=FComboBox;
end;

procedure TCustomComboBoxMediator.SetupControl;
begin
  inherited SetupControl;
  FCombobox.OnEditingDone:=@ViewChangedHandler;
  If FUseItemIndex then
    FComboBox.Style:=csDropDownList; // Force
end;

function TCustomComboBoxMediator.DefaultUseItemindex: Boolean;
begin
  Result:=False;
end;

constructor TCustomComboBoxMediator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FUseItemIndex:=DefaultUseItemindex;
  CheckViewPropertyName;
end;

procedure TCustomComboBoxMediator.CheckViewPropertyName;

begin
  If FUseItemindex then
    begin
    ViewPropertyName:='ItemIndex';
    If Assigned(FComboBox) then
    FComboBox.Style:=csDropDownList; // Force
    end
  else
    ViewPropertyName:='Text';
end;

procedure TCustomComboBoxMediator.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  If (Operation=opRemove) and (AComponent=FCombobox) then
    begin
    FCombobox:=Nil;
    ViewChanged;
    end;
end;

class function TCustomComboBoxMediator.ViewClass: TClass;
begin
  Result:=TComboBox;
end;

{ TMemoMediator }

function TMemoMediator.GetView: TObject;
begin
  Result:=FMemo;
end;

procedure TMemoMediator.SetMemo(const AValue: TMemo);
begin
  SetControl(FMemo,AValue);
end;

procedure TMemoMediator.SetupControl;
begin
  inherited SetupControl;
  FMemo.OnChange:=@ViewChangedHandler;
  FMemo.Lines.Clear;
end;

procedure TMemoMediator.SetSubject(const AValue: TObject);
begin
  inherited SetSubject(AValue);
  CheckPropertyType;
end;

procedure TMemoMediator.CheckPropertyType;

Var
  K : TTypeKind;
  O : TObject;

begin
  If Assigned(Subject) then
    begin
    K:=PropType(Subject,SubjectPropertyName);
    If Not (FStringsProperty or (K in [tkClass,tkSString,tkLString,tkAString,tkChar,tkwstring])) then
      Raise EObserver.CreateFmt(SErrInvalidPropertyKind,[Self.Name,SubjectPropertyName]);
    FStringsProperty:=(K=tkClass);
    If FStringsProperty then
      begin
      O:=GetObjectProp(Subject,SubjectPropertyName);
      If Assigned(O) and Not (O is TStrings) then
        Raise EObserver.CreateFmt(SErrInvalidPropertyKind,[Self.Name,SubjectPropertyName]);
      end;
    end;
end;

procedure TMemoMediator.SetSubjectPropertyName(const AValue: String);
begin
  inherited SetSubjectPropertyName(AValue);
  CheckPropertyType;
end;

constructor TMemoMediator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ViewPropertyName:='Lines';
end;

procedure TMemoMediator.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  If (Operation=opRemove) and (AComponent=FMemo) then
    begin
    FMemo:=Nil;
    ViewChanged;
    end;
end;

class function TMemoMediator.ViewClass: TClass;
begin
  Result:=TMemo;
end;

procedure TMemoMediator.DoViewToObject;

begin
  If FStringsProperty then
    SetObjectProp(Subject,SubjectPropertyName,FMemo.Lines)
  else
    SetStrProp(Subject,SubjectPropertyName,FMemo.Lines.text);
end;

procedure TMemoMediator.DoObjectToView;
begin
  If FStringsProperty then
    FMemo.Lines:=GetObjectProp(Subject,SubjectPropertyName,TStrings) as TStrings
  else
    FMemo.Lines.Text:=GetStrProp(Subject,SubjectPropertyName);
end;

{ TItemComboBoxMediator }

function TItemComboBoxMediator.DefaultUseItemindex: Boolean;
begin
  Result:=True;
end;

{ TFileNameEditMediator }

constructor TFileNameEditMediator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ViewPropertyName:='FileName';
end;

class function TFileNameEditMediator.ViewClass: TClass;
begin
  Result:=TFileNameEdit;
end;

{ TDirectoryEditMediator }

constructor TDirectoryEditMediator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ViewPropertyName:='Directory';
end;

class function TDirectoryEditMediator.ViewClass: TClass;
begin
  Result:=TDirectoryEdit;
end;

{ TDateEditMediator }

procedure TDateEditMediator.SetEdit(const AValue: TDateEdit);
begin
  SetControl(FEdit,AValue);
end;

function TDateEditMediator.GetView: TObject;
begin
  Result:=FEdit;
end;

procedure TDateEditMediator.SetupControl;
begin
  inherited SetupControl;
  FEdit.OnChange:=@ViewChangedHandler;
  FEdit.Clear;
end;

procedure TDateEditMediator.CheckDisableControl;
begin
  inherited CheckDisableControl;
  If Not FEdit.Enabled then
    FEdit.Clear;
end;

constructor TDateEditMediator.Create(AOwner: TComponent);
begin
  ViewPropertyName:='Text';
  inherited Create(AOwner);
end;

destructor TDateEditMediator.Destroy;
begin
  if Assigned(FEdit) then
    FEdit.OnChange := nil;
  inherited Destroy;
end;

procedure TDateEditMediator.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  If (Operation=opRemove) and (AComponent=FEdit) then
    begin
    FEdit:=Nil;
    ViewChanged;
    end;
end;

class function TDateEditMediator.ViewClass: TClass;
begin
  Result:=TDateEdit;
end;

{ TFloatSpinEditMediator }

procedure TFloatSpinEditMediator.SetSpinEdit(const AValue: TFloatSpinEdit);
begin
  SetControl(FSpinEdit,AValue);
end;

function TFloatSpinEditMediator.GetView: TObject;
begin
  Result:=FSpinEdit;
end;

procedure TFloatSpinEditMediator.SetupControl;
begin
  inherited SetupControl;
  FSpinEdit.OnChange:=@ViewChangedHandler;
  FSpinEdit.Clear;
end;

constructor TFloatSpinEditMediator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ViewPropertyName:='Value';
end;

procedure TFloatSpinEditMediator.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  If (Operation=opRemove) and (AComponent=FSpinEdit) then
    begin
    FSpinEdit:=Nil;
    ViewChanged;
    end;
end;

class function TFloatSpinEditMediator.ViewClass: TClass;
begin
  Result:=TFloatSpinEdit;
end;

{ TSpeedButtonMediator }

procedure TSpeedButtonMediator.SetButton(const AValue: TSpeedbutton);
begin
  SetControl(FButton,AValue);
end;

procedure TSpeedButtonMediator.DoObjectToView;
begin
  FButton.Down:=GetBooleanValue;
end;

procedure TSpeedButtonMediator.DoViewToObject;
begin
  SetBooleanValue(FButton.Down);
end;

function TSpeedButtonMediator.GetBooleanValue: Boolean;
begin
  Result:=False;
end;

procedure TSpeedButtonMediator.SetBooleanValue(AValue: Boolean);
begin
  // Do nothing
end;

constructor TSpeedButtonMediator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ViewPropertyName:='Down';
end;

procedure TSpeedButtonMediator.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  If (Operation=opRemove) and (AComponent=FButton) then
    begin
    FButton:=Nil;
    ViewChanged;
    end;

end;


function TSpeedButtonMediator.GetView: TObject;
begin
  Result:=FButton;
end;

procedure TSpeedButtonMediator.SetupControl;
begin
  inherited SetupControl;
  FButton.OnClick:=@ViewChangedHandler;
end;


{ TRadioGroupMediator }

procedure TRadioGroupMediator.SetRadioGroup(const AValue: TRadioGroup);
begin
  SetControl(FRadioGroup,AValue);
end;

function TRadioGroupMediator.GetView: TObject;
begin
  Result:=FRadioGroup;
end;

procedure TRadioGroupMediator.SetupControl;
begin
  inherited SetupControl;
  FRadioGroup.OnClick:=@ViewChangedHandler;
end;

procedure TRadioGroupMediator.DoObjectToView;
begin
  FRadioGroup.ItemIndex:=GetOrdProp(Subject,SubjectPropertyName);
end;

procedure TRadioGroupMediator.DoViewToObject;
begin
  SetOrdProp(Subject,SubjectPropertyName,FRadioGroup.ItemIndex);
end;

constructor TRadioGroupMediator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ViewPropertyName:='ItemIndex';
end;

procedure TRadioGroupMediator.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  If (Operation=opRemove) and (AComponent=FRadioGroup) then
    begin
    FRadioGroup:=Nil;
    ViewChanged;
    end;
end;

class function TRadioGroupMediator.ViewClass: TClass;
begin
  Result:=TRadioGroup;
end;

{ TColorButtonMediator }

procedure TColorButtonMediator.SetColorButton(const AValue: TColorButton);
begin
  SetControl(FColorButton,AValue);
end;

function TColorButtonMediator.GetView: TObject;
begin
  Result:=FColorButton;
end;

procedure TColorButtonMediator.SetupControl;
begin
  inherited SetupControl;
  FColorButton.OnColorChanged:=@ViewChangedHandler;
end;


constructor TColorButtonMediator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ViewPropertyName:='ButtonColor';
end;

procedure TColorButtonMediator.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  If (Operation=opRemove) and (AComponent=FColorButton) then
    begin
    FColorButton:=Nil;
    ViewChanged;
    end;
end;

class function TColorButtonMediator.ViewClass: TClass;
begin
  Result:=TColorButton;
end;

{ TImageMediator }


procedure TImageMediator.SetImage(const AValue: TImage);
begin
  SetControl(FImage,AValue);
end;

procedure TImageMediator.SetCenteredProperty(const AValue: String);
begin
  if FCenteredProperty=AValue then exit;
  FCenteredProperty:=AValue;
  MaybeObjectToView;
end;

procedure TImageMediator.SetStretchedProperty(const AValue: String);
begin
  if FStretchedProperty=AValue then exit;
  FStretchedProperty:=AValue;
  MaybeObjectToView;
end;

procedure TImageMediator.DoObjectToView;

Var
  O : TObject;

begin
  If (FCenteredProperty<>'') then
    FImage.Center:=GetOrdProp(Subject,FCenteredProperty)=Ord(True);
  If (FStretchedProperty<>'') then
    FImage.Stretch:=GetOrdProp(Subject,FStretchedProperty)=Ord(True);
  O:=GetObjectProp(Subject,SubjectPropertyName);
  If O is TPicture then
    FImage.Picture:=O as TPicture
  else if O is TGraphic then
    FImage.Picture.Graphic.Assign(O as TGraphic)
  else if O is TStream then
    FImage.Picture.Graphic.LoadFromStream(O as TStream)
  else if (O<>Nil) then
    Raise EObserver.CreateFmt(SErrUnknownImageClass,[O.ClassName]);

end;

procedure TImageMediator.DoViewToObject;

Var
  O : TObject;

begin
  O:=GetObjectProp(Subject,SubjectPropertyName);
  If O is TPicture then
    SetObjectProp(Subject,SubjectPropertyName,FImage.Picture)
  else if O is TGraphic then
    SetObjectProp(Subject,SubjectPropertyName,FImage.Picture.Graphic)
  else if O is TStream then
    FImage.Picture.Graphic.SaveToStream(O as TStream)
  else if (O<>Nil) then
    Raise EObserver.CreateFmt(SErrUnknownImageClass,[O.ClassName]);
end;

function TImageMediator.GetView: TObject;
begin
  Result:=FImage;
end;

procedure TImageMediator.SetupControl;
begin
  inherited SetupControl;
  FImage.OnPictureChanged:=@ViewChangedHandler;
end;

constructor TImageMediator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ViewPropertyName:='Picture';
end;

procedure TImageMediator.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  If (Operation=opRemove) and (AComponent=FImage) then
    begin
    FImage:=Nil;
    ViewChanged;
    end;
end;

{ TObjectComboBoxMediator }

function TObjectComboBoxMediator.GetDisplayFieldName: String;
begin
  Result:=FDisplayFieldName;
  If Result='' then
    Result:='Caption';
end;

procedure TObjectComboBoxMediator.SetComboBox(const AValue: TComboBox);
begin
  SetControl(FComboBox,AValue);
end;

procedure TObjectComboBoxMediator.SetDisplayFieldName(const AValue: String);
begin
  FDisplayFieldName:=AValue;
  ViewChanged;
end;

function TObjectComboBoxMediator.GetView: TObject;
begin
  Result:=FCOmbobOx;
end;

procedure TObjectComboBoxMediator.ValueListChanged;
begin
  ViewChanged;
end;

procedure TObjectComboBoxMediator.SetupControl;

Var
  DFN,N : String;
  O,OS : TObject;
  I,II : Integer;

begin
  inherited SetupControl;
  DFN:=DisplayFieldName;
  If FComboBox.Enabled then
    FCombobox.Enabled:=Assigned(ValueList);
  FCombobox.Style:=csDropDownList;
  FCombobox.Items.BeginUpdate;
  try
    FCombobox.Clear;
    If Assigned(Subject) and (SubjectPropertyName<>'') then
      OS:=GetObjectProp(Subject,SubjectPropertyName)
    else
      OS:=Nil;
    II:=-1;
    If Assigned(ValueList) then
      For I:=0 to ValueList.Count-1 do
        begin
        O:=ValueList[i];
        If Not IsPublishedProp(O,DFN) then
          Raise EObserver.CreateFmt(SErrUnknownProperty,[DFN,O.ClassName]);
        If (O<>Nil) and (O=OS) then
          II:=I;
        N:=GetStrProp(O,DFN);
        Control.Items.AddObject(N,O);
        end;
    FComboBox.ItemIndex:=II;
    If (FComboBox.Items.Count>1) and (FComboBox.Items.Count<21) then
      FComboBox.DropDownCount:=FComboBox.Items.Count;
  finally
    FCombobox.Items.EndUpdate;
  end;
  FComboBOx.OnEditingDone:=@ViewChangedHandler;
end;

procedure TObjectComboBoxMediator.DoObjectToView;

Var
  O : TObject;

begin
  O:=GetObjectProp(Subject,SubjectPropertyName);
  If (O=Nil) then
    FCombobox.ItemIndex:=-1
  else
    FCombobox.ItemIndex:=FCombobox.Items.IndexOfObject(O);
end;

procedure TObjectComboBoxMediator.DoViewToObject;

Var
  O : TObject;

begin
  With FCombobox do
    If (ItemIndex=-1) then
      O:=Nil
    else
      O:=Items.Objects[ItemIndex];
  SetObjectProp(Subject,SubjectPropertyName,O);
end;

constructor TObjectComboBoxMediator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ViewPropertyName:='ItemIndex';
end;

procedure TObjectComboBoxMediator.RefreshList;
begin
  ValueListChanged;
end;

{ TListViewItemMediator }

procedure TListViewItemMediator.ObjectToView;

Var
  I : integer;
  Prop,PropVal : string;

begin
  ListItem.SubItems.BeginUpdate;
  try
    ListItem.SubItems.Clear;
    for I:=0 to FieldsInfo.Count-1 do
      begin
      Prop:=FieldsInfo[I].PropertyName;
      PropVal:=GetPropValue(Subject,Prop,True);
      if Assigned(OnBeforeSetupField) then
        OnBeforeSetupField(Subject,Prop,PropVal);
      If (I=0) Then
        ListItem.Caption:=PropVal
      else
        ListItem.SubItems.Add(PropVal);
      end;
  Finally
    ListItem.SubItems.BeginUpdate;
  end;
end;

{ TListViewMediator }

procedure TListViewMediator.SetListView(const AValue: TListView);
begin
  if FListView=AValue then exit;
  If Assigned(FListView) then
    FListView.RemoveFreeNotification(Self);
  FListView:=AValue;
  If Assigned(FListView) then
    FListView.FreeNotification(Self);
  MaybeObjectToView;
end;

function TListViewMediator.GetSelectedObject: TObject;

Var
  LI : TListItem;

begin
  Result:=Nil;
  LI:=FListView.Selected;
  If Assigned(Li) and Assigned(LI.Data) then
    Result:=TListItemMediator(Li.Data).Subject;
end;


procedure TListViewMediator.SetSelectedObject(const AValue: TObject);

Var
  O : TListViewItemMediator;
  I : Integer;

begin
  O:=TListViewItemMediator(FIndObjectMediator(AValue,I));
  IF O<>Nil then
    FListView.Selected:=O.ListItem;
end;

function TListViewMediator.CreateItemMediator(AData: TObject; ARow: integer
  ): TListItemMediator;

Var
  LIM : TListViewItemMediator;

begin
  LIM:=TListViewItemMediator.Create;
  LIM.FListItem:=FListView.Items.Add;
  LIM.FListItem.Data:=Lim;
  Result:=Lim;
end;

procedure TListViewMediator.DoDeleteItemMediator(AIndex: Integer;
  AMediator: TListItemMediator);

Var
  Li : TListItem;

begin
  LI:=TListViewItemMediator(AMediator).FListItem;
  TListViewItemMediator(AMediator).FListItem:=Nil;
  FreeAndNil(Li);
end;


function TListViewMediator.GetView: TObject;
begin
  Result:=FListView;
end;

procedure TListViewMediator.CreateColumns;

Var
  I : Integer;
  C : TListColumn;
  FI : TMediatorFieldInfo;

begin
  FListView.Columns.BeginUpdate;
  try
    FListView.Columns.Clear;
    For I:=0 to FieldsInfo.Count-1 do
      begin
      FI:=FieldsInfo[i];
      C:=FListView.Columns.Add;
      C.Caption:=FI.Caption;
      C.Width:=FI.Width;
      C.Alignment:=FI.Alignment;
      end;
  Finally
    FListView.Columns.EndUpdate;
  end;
end;

procedure TListViewMediator.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  If (Operation=opRemove) and (AComponent=FListView) then
    FListView:=Nil;
end;

end.

