unit objecteditor; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  ExtCtrls, ButtonPanel, ComCtrls, Spin, StdCtrls, Buttons, fpobserver,
  lclmediators, cdcovers, typinfo, fracoverobject;

type

  { TFrameSheet }

  TFrameSheet = Class(TTabSheet)
  private
    FCoverObjectFrame: TCoverObjectFrame;
  Public
    Property ObjectFrame : TCoverObjectFrame Read FCoverObjectFrame Write FCoverObjectFrame;
  end;

  { TObjectEditorForm }

  TObjectEditorForm = class(TForm)
    ButtonPanel1: TButtonPanel;
    PCObject: TPageControl;
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormDestroy(Sender: TObject);
  private
    FCoverObject: TCoverObject;
    FMemento : TMemento;
    procedure SetCoverObject(const AValue: TCoverObject);
    { private declarations }
  public
    { public declarations }
    Property CoverObject : TCoverObject Read FCoverObject Write SetCoverObject;
  end;

var
  ObjectEditorForm: TObjectEditorForm;

implementation

uses
  fraobjectpos,
  fracoverlabel,
  fracovershape,
  fracoverimage,
  fratracklist;


{ TObjectEditorForm }


procedure TObjectEditorForm.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FMemento);
end;


procedure TObjectEditorForm.FormClose(Sender: TObject;
  var CloseAction: TCloseAction);
begin
  If (ModalResult<>mrOK) and Assigned(FCoverObject) then
    FCoverObject.ApplyMemento(FMemento);
end;

procedure TObjectEditorForm.SetCoverObject(const AValue: TCoverObject);

  Function CreateSheet(AClass : TObjectCoverFrameClass) : TCoverObjectFrame;

  Var
    S : TFrameSheet;

  begin
    S:=TFrameSheet.Create(Self);
    Result:=AClass.Create(S);
    S.Caption:=Result.TabCaption;
    S.ObjectFrame:=Result;
    S.Parent:=PCObject;
    Result.Parent:=S;
    Result.Align:=alClient;
    Result.Visible:=True;
  end;

Var
  I : Integer;
  F : TCoverObjectFrame;

begin
  if FCoverObject=AValue then exit;
  FCoverObject:=AValue;
  FMemento:=FCoverObject.GetMemento;
  CreateSheet(TObjectPosFrame);
  If FCoverObject is TLabelObject then
    CreateSheet(TCoverLabelFrame)
  else if FCoverObject is TShapeObject then
    CreateSheet(TCoverShapeFrame)
  else if FCoverObject is TImageObject then
    CreateSheet(TImageObjectFrame)
  else if FCoverObject is TTrackList then
    CreateSheet(TTrackListFrame);
  For I:=0 to PCObject.PageCount-1 do
    If PCObject.Pages[i] is TFrameSheet then
      begin
      F:=TFrameSheet(PCObject.Pages[i]).ObjectFrame;
      F.InitFrame;
      F.CoverObject:=FCoverObject;
      end;
  PCObject.ActivePageIndex:=PCObject.PageCount-1;
end;




initialization
  {$I objecteditor.lrs}

end.

